/***************************************************************************
                     cannyfilter.cpp  -  description
                            -------------------
    begin                : Tue Feb 19 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/


#include "cannyfilter.h"

#include <vigra/stdimage.hxx>
#include <vigra/edgedetection.hxx>
#include <vigra/imageiterator.hxx>

using namespace std;
using namespace vigra;



//////////////////////////////// Public Functions ////////////////////////////////

BEGIN_CONFIG(cannyFilter,vistaLab::module)
    CONFIGINPUTPIN (m_inputImage     , __("Input Image"                       ))
    CONFIGOUTPUTPIN(m_edgeImage      , __("Output Image"                      ))
    CONFIGPARAMETER(m_scale          , __("Scale factor for exponential image"))
    CONFIGPARAMETER(m_threshold      , __("Threshold for valid edges"         ))
    CONFIGPARAMETER(m_edgeIndicator  , __("Intensity indicator for edges"     ))
    CONFIGPARAMETER(m_noEdgeIndicator, __("Intensity indicator for non-edges" ))
END_CONFIG()

/***************************************************************************/
/*!
 *  Constructor sets up provided colorspaces and frameRates. The filter can
 *  process an arbitrary number of frames
 *
 ***************************************************************************/
cannyFilter::cannyFilter() :
vistaLab::module(PROCESSING,__("Filter"),__("Canny"),
__("Locates Edges using the Canny filter"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_scale    (4.0),
m_threshold(4.0),
m_edgeIndicator(255),
m_noEdgeIndicator(0)
{
}


/***************************************************************************/
/*!
 *  Up to now there is no initialization
 *
 ***************************************************************************/
bool cannyFilter::init()
{
    return true;
}


/***************************************************************************/
/*!
 *  Up to now there is no cleaning up
 *
 ***************************************************************************/
void cannyFilter::done()
{
}


/***************************************************************************/
/*!
 *  Up to now there is no processing
 *
 ***************************************************************************/
bool cannyFilter::process()
{
    m_edgeImage = new BImage(m_inputImage->width(),
        m_inputImage->height(),(unsigned char) m_noEdgeIndicator );

    cannyEdgeImage (srcImageRange(*m_inputImage),destImage(*m_edgeImage),
        m_scale, m_threshold, (unsigned char) m_edgeIndicator);// marker for destination image

    return true;
}
