/***************************************************************************
                       chronostop.cpp  -  description
                            -------------------
    begin                : Tue Feb 19 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "chronostop.h"

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include <wx/stattext.h>
#include <wx/statbox.h>
#include <wx/sizer.h>
#include <wx/dialog.h>


using namespace std;


//////////////////////////// Public Functions /////////////////////////////

BEGIN_CONFIG(chronoStop,vistaLab::module)
    CONFIGINPUTPIN (m_stopWatch   ,__("Stop Watch")                    )
    CONFIGPARAMETER(m_meanInterval,__("Number of frames for mean time"))
END_CONFIG()

/***************************************************************************/
/*!
 *
 *
 ***************************************************************************/
chronoStop::chronoStop() :
vistaLab::module(GUI,__("Miscellaneous"),__("Stopwatch Stopper"),
__("Used together with Stopwatch Starter to measure processing time"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_meanInterval(10),
m_outputWindow(0)
{
}


/***************************************************************************/
/*!
 *  If there is an output window, it gets deleted in the destructor
 *
 ***************************************************************************/
chronoStop::~chronoStop()
{
    if(m_outputWindow)
        m_outputWindow->Destroy();
}


/***************************************************************************/
/*!
 *  A new output window is created if active
 *
 ***************************************************************************/
bool chronoStop::init()
{
    // First, remove old one if there is one
    if(m_outputWindow){
        delete m_outputWindow;
        m_outputWindow = 0;
    }

    // Then, create a new one if active
    m_outputWindow = new wxDialog(NULL,-1,
        wxString(getInstanceName().c_str()));
    m_topSizer     = new wxBoxSizer(wxHORIZONTAL);

    m_outputWindow->SetSizer(m_topSizer);

    m_labelSizer = new wxStaticBoxSizer(new wxStaticBox(m_outputWindow,-1,
        getInstanceName().c_str()),wxHORIZONTAL);
    m_timeLabel = new wxStaticText(m_outputWindow,-1,wxEmptyString);

    m_labelSizer->Add(50,0,1,wxEXPAND);
    m_labelSizer->Add(m_timeLabel,wxSizerFlags(0).Centre().Border(wxALL));
    m_labelSizer->Add(50,0,1,wxEXPAND);

    m_topSizer->Add(m_labelSizer,wxSizerFlags(1).Centre().Border(wxALL));
    m_topSizer->Fit(m_outputWindow);

    m_outputWindow->Centre();
    m_outputWindow->Show(true);

    m_meanTime.resize(m_meanInterval);
    m_frameCounter=0;

    return true;
}


/***************************************************************************/
/*!
 *  There is nothing to clean up - the window has to stay and is destroyed
 *  in the destructor (or in a new init() - call)
 *
 ***************************************************************************/
void chronoStop::done()
{
}


/***************************************************************************/
/*!
 *  The processing time between the start module and this one gets captured
 *
 ***************************************************************************/
bool chronoStop::process()
{
    m_time = m_stopWatch->Time();
    // Set up the time label

    m_meanTime[m_frameCounter%m_meanInterval]=m_time;
    long m_mean=0;

    int count=std::min(m_frameCounter+1,m_meanInterval);

    m_frameCounter++;

    for(int i=0;i<count;++i)
        m_mean+=m_meanTime[i];

    if (m_time != 0)
        m_timeLabel->SetLabel(
        wxString::Format(_("Current: %5ld ms -> max. possible frame rate:\
 %3.2f frames/sec\nMean: %3.5f ms."),
                           m_time, 1000.0 / (double) m_time, m_mean / (double) count ));
    else
        m_timeLabel->SetLabel(
        _(" < 1 ms -> max. possible frame rate:\
 > 1000 frames/sec"));

    // Layout new label
    m_labelSizer->SetItemMinSize(m_timeLabel,m_timeLabel->GetBestSize());
    m_topSizer->Fit(m_outputWindow);
    m_outputWindow->Layout();
    return true;
}
