/***************************************************************************
                        chronostop.h  -  description
                            -------------------
    begin                : Tue Feb 19 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _CHRONOSTOP_H_
#define _CHRONOSTOP_H_

#include "vlmodule.h"
#include <wx/timer.h>

class wxDialog;
class wxStaticText;
class wxBoxSizer;
class wxStaticBoxSizer;


/***************************************************************************/
//! Used in conjunction with chronoStart to measure processing time
/*!
 *  To measure processing time (i.e. the time of a set of process() - calls)
 *  a pair of chronoStart and chronoStop - modules may be used.
 *  They have to be placed before and after the modules which should be
 *  measured. The chronoStop - module must select the corresponding
 *  start - module in its configuration dialog. If drawing of the
 *  stop - module is active, the consumed time (and the resulting framerate
 *  which would be possible for realtime processing) is displayed in a
 *  child window.
 *
 *  As the time is measured in the process() - functions, only processing
 *  gets counted - i.e. all draw() - calls of all modules are not taken into
 *  account as they happen after processing in vistaLabProcessor::process().
 *
 * \ingroup Modules
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class chronoStop : public vistaLab::module {
    VISTALABMODULE(chronoStop);
public:
    chronoStop();
    virtual ~chronoStop();
    virtual bool init();
    virtual void done();
    virtual bool process();

private:
    INPUTPIN (wxStopWatch, m_stopWatch   );
    PARAMETER(int        , m_meanInterval);

    // The time value
    long m_time;

    // GUI members
    wxDialog         *m_outputWindow;
    wxStaticText     *m_timeLabel;
    wxBoxSizer       *m_topSizer;
    wxStaticBoxSizer *m_labelSizer;

    std::vector< long > m_meanTime;
    int                 m_frameCounter;
};

#endif // #ifndef _CHRONOSTOP_H_
