/***************************************************************************
                      deinterlacer.cpp  -  description
                            -------------------
    begin                : Mon Jun 3 2002
    copyright            : (C) 2002 by wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "deinterlacer.h"

#include <vigra/stdimage.hxx>
#include <vigra/resizeimage.hxx>
#include <vigra/imageiterator.hxx>
#include <vigra/copyimage.hxx>

using namespace std;



//////////////////////////// Public Functions /////////////////////////////

BEGIN_CONFIG(deinterlacer,vistaLab::module)
    CONFIGINPUTPIN (m_inputImage       ,__("Input Image")                    )
    CONFIGOUTPUTPIN(m_deinterlacedImage,__("Deinterlaced Image")             )
    CONFIGPARAMETER(m_fillEmptyLines   ,__("Fill empty lines with old field"))
END_CONFIG()


/***************************************************************************/
/*!
 *  Constructor sets up default values
 *
 ***************************************************************************/
deinterlacer::deinterlacer() :
vistaLab::module(PROCESSING,__("Geometry"),__("Image Deinterlacer"),
__("Doubles height and fills new lines "),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_fillEmptyLines(false)
{
}


/***************************************************************************/
/*!
 *  No initialization necessary
 *
 ***************************************************************************/
bool deinterlacer::init()
{
    return true;
}


/***************************************************************************/
/*!
 *  No cleaning up necessary
 *
 ***************************************************************************/
void deinterlacer::done()
{
}


/***************************************************************************/
/*!
 *
 *
 ***************************************************************************/
bool deinterlacer::process()
{
    // Initialize empty new Image
    m_deinterlacedImage = new
        vigra::BRGBImage(m_inputImage->width(),2*m_inputImage->height());

    //If filling empty lines, deinterlacing is simply resizing
    if(m_fillEmptyLines)
        vigra::resizeImageNoInterpolation(srcImageRange(*m_inputImage),
                                    destImageRange(*m_deinterlacedImage));
    else
    {
        vigra::StridedImageIterator< vigra::BRGBImage::value_type >
           dest(&(*m_deinterlacedImage->begin()), m_deinterlacedImage->width(),1,2);
        copyImage(m_inputImage->upperLeft(),m_inputImage->lowerRight(),
                  m_inputImage->accessor(),dest,m_deinterlacedImage->accessor());
    }


    return true;
}
