/***************************************************************************
                           grayimagenormalizer.cpp  -  description
                            -------------------
         begin                : Wed, Jul 17 2002
         copyright            : (C) 2002 by Fabian Wenzel
         email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "grayimagenormalizer.h"

#include <vigra/inspectimage.hxx>
#include <vigra/transformimage.hxx>

using namespace vigra;

////////////////////////// Public Functions ///////////////////////////////

BEGIN_CONFIG(grayImageNormalizer,vistaLab::module)
     CONFIGINPUTPIN  (m_inputImage     , __("Input image" ))
     CONFIGOUTPUTPIN (m_normalizedImage, __("Output image"))
END_CONFIG()

/***************************************************************************/
/*!
 *
 *
 ***************************************************************************/
grayImageNormalizer::grayImageNormalizer ():
vistaLab::module (PROCESSING,__("Enhancement"),__("Grayimage Normalizer"),
__("Normalizes the intensity values of a gray image"),
"Fabian Wenzel","wenzel@tu-harburg.de")
{
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
bool grayImageNormalizer::init ()
{
    return true;
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
void grayImageNormalizer::done ()
{
}


/***************************************************************************/
/*!
 *  Normalizes the image
 *
 ***************************************************************************/
bool grayImageNormalizer::process()
{
    m_normalizedImage = new BImage(m_inputImage->size());

    // Find gray range
    FindMinMax<BImage::PixelType> minmax;
    inspectImage(srcImageRange(*m_inputImage), minmax);

    // transform to range 0...255
    transformImage(srcImageRange(*m_inputImage), destImage(*m_normalizedImage),
                   linearIntensityTransform(
                   255.0 / (minmax.max - minmax.min), // scaling
                          - minmax.min));             // offset
    return true;
}
