/***************************************************************************
                     harriscorners.cpp  -  description
                            -------------------
    begin                : Wed Mar 29 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de
 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "harriscorners.h"

#include <vigra/imageiterator.hxx>
#include <vigra/cornerdetection.hxx>
#include <vigra/transformimage.hxx>
#include <vigra/localminmax.hxx>
#include <vigra/rgbvalue.hxx>

#include <climits>
#include <complex>

using namespace std;
using namespace vigra;



////////////////////////// Public Functions ////////////////////////////////


BEGIN_CONFIG(harrisCorners,vistaLab::module)
    CONFIGINPUTPIN (m_inputImage , __("Input Image" ))
    CONFIGOUTPUTPIN(m_outputImage, __("Corner Image"))
END_CONFIG()


/***************************************************************************/
/*!
 *  Constructor sets up needed colorspaces etc (which is RGB only)
 *
 ***************************************************************************/
harrisCorners::harrisCorners () :
vistaLab::module (PROCESSING,__("Corners"),__("Plessey Corners"),
__("Calculates a corner image using the plessey detector"),
"Fabian Wenzel","wenzel@tu-harburg.de")
{
}


/***************************************************************************/
/*!
 *  Does nothing as there is nothing which has to be initialized
 *
 ***************************************************************************/
bool harrisCorners::init ()
{
    return true;
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
void harrisCorners::done ()
{
}


/***************************************************************************/
/*!
 *  Calls the Corner Response Function of VIGRA's implementation of the
 *  plessey detector
 *
 ***************************************************************************/
bool harrisCorners::process()
{
    // Prepare destination image
    m_outputImage = new vigra::BImage(m_inputImage->size());

    // Intermediate image needed for float values of corner response
    vigra::FImage cornerResponse(m_inputImage->size());

    // find corner response at scale 1.0
    vigra::cornerResponseFunction(m_inputImage->upperLeft(),m_inputImage->lowerRight(),
        vigra::RGBToGrayAccessor< BRGBImage::PixelType >(),
        cornerResponse.upperLeft(),cornerResponse.accessor(), 1.0);

    // find local maxima of corner response, mark with 1
    vigra::localMaxima(srcImageRange(cornerResponse),destImage(*m_outputImage));

    // threshold corner response to keep only strong corners (above 400.0)
    transformImage(srcImageRange(cornerResponse), destImage(cornerResponse),
        vigra::Threshold< double , double >( 400.0,
        100000000.0, 0.0, 255.0)); 

    // combine thresholding and local maxima
    vigra::combineTwoImages(srcImageRange(*m_outputImage),
        srcImage(cornerResponse),
        destImage(*m_outputImage),std::multiplies<float>());

    return true;
}
