/***************************************************************************
                      imagechannel.cpp  -  description
                            -------------------
    begin                : Thu Jul 18 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "imagechannel.h"

#include "vlapp.h"
#include "vlframe.h"
#include "vlview.h"
#include "vlchannelwindow.h"

#include <vigra/stdimage.hxx>


using namespace std;

//////////////////////////// Public Functions /////////////////////////////

BEGIN_CONFIG(imageChannel,vistaLab::module)
    CONFIGINPUTPIN(m_inputImage,__("Input Image"))
END_CONFIG()

/***************************************************************************/
/*!
 *  Registers an RGB input image
 *
 ***************************************************************************/
imageChannel::imageChannel() :
vistaLab::module(GUI,__("Output"),__("Image Channel"),
__("Shows an RGB image in the main view"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_channelWindow(0)
{
}


/***************************************************************************/
/*!
 *  Copy Constructor copies the image slot from the source and registers a
 *  new channel window in the main GUI (this is the constructor which
 *  is called when adding an active module)
 *
 ***************************************************************************/
imageChannel::imageChannel(const imageChannel &src) : vistaLab::module(src)
{
    COPYPIN(m_inputImage,src);

    m_channelWindow = ((vistaLab::frame *) wxGetApp().GetTopWindow())->
        getChannelView()->addChannel(this);
}

/***************************************************************************/
/*!
 *  Deletes the associated channel window
 *
 ***************************************************************************/
imageChannel::~imageChannel()
{
    //Only do something if we are not the module in the repository and
    //if we are not shutting down, otherwise wxWidgets will delete the
    //childwindow itself
    if(m_channelWindow){
        ((vistaLab::frame *) wxGetApp().GetTopWindow())->
            getChannelView()->removeChannel(m_channelWindow);
        m_channelWindow->Destroy();
    }
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
bool imageChannel::init()
{
    return true;
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
void imageChannel::done()
{
}


/***************************************************************************/
/*!
 *  Hands the output image to the channelWindow
 *
 ***************************************************************************/
bool imageChannel::process()
{
    m_channelWindow->eatImage(*m_inputImage);
    return true;
}


/***************************************************************************/
/*!
 *  Hands the output image to the channelWindow
 *
 ***************************************************************************/
void imageChannel::postConfig()
{
    if(m_channelWindow) // only update if we are configuring the active module
        m_channelWindow->updateInstanceName();
}
