/***************************************************************************
                   imagemultiplier.cpp  -  description
                            -------------------
    begin                : Fri Aug 30 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "imagemultiplier.h"
#include <vigra/combineimages.hxx>
#include <vigra/functorexpression.hxx>

using namespace vigra;

////////////////////////////// Public Functions /////////////////////////////

BEGIN_CONFIG(imageMultiplier,vistaLab::module)
    CONFIGINPUTPIN (m_firstImage  ,__("First image"     ))
    CONFIGINPUTPIN (m_secondImage ,__("Second image"    ))
    CONFIGOUTPUTPIN(m_outputImage ,__("Multiplied image"))
END_CONFIG()


/***************************************************************************/
/*!
 *  Constructor sets up default bufferLength
 *
 ***************************************************************************/
imageMultiplier::imageMultiplier() :
vistaLab::module(PROCESSING,__("Miscellaneous"),__("Grayimage Multiplier"),
__("Multiplies two images pixelwise"),
"Fabian Wenzel","wenzel@tu-harburg.de")
{
}


/***************************************************************************/
/*!
 *  No initialization
 *
 ***************************************************************************/
bool imageMultiplier::init()
{
    return true;
}


/***************************************************************************/
/*!
 *  No cleaning up
 *
 ***************************************************************************/
void imageMultiplier::done()
{
}


/***************************************************************************/
/*!
 *  Multiplies two images
 *
 ***************************************************************************/
bool imageMultiplier::process()
{
    if(m_firstImage->width()  != m_secondImage->width() ||
       m_firstImage->height() != m_secondImage->height()){

        vistaLab::setError(_("Images are of different size"));
        return false;
    }

    m_outputImage = new BImage(m_firstImage->width(),m_firstImage->height());

    using namespace vigra::functor;

    combineTwoImages(srcImageRange(*m_firstImage),srcImage(*m_secondImage),
                     destImage(*m_outputImage),Arg1()*Arg2());

    return true;
}
