/***************************************************************************
                  imagesequencewriter.cpp  -  description
                            -------------------
    begin                : Wed, Apr 23 2003
    copyright            : (C) 2003 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include <wx/image.h>

#include "imagesequencewriter.h"

#include <cstdio>


///////////////////////////// Public Functions ///////////////////////////////

std::vector< int >         imageSequenceWriter::m_imageTypes;
std::vector< std::string > imageSequenceWriter::m_imageTypeDescriptions;
std::vector< std::string > imageSequenceWriter::m_imageTypeSuffixes;

BEGIN_CONFIG(imageSequenceWriter,vistaLab::module)
    CONFIGPARAMETER  (m_quality,   __("Quality 1-1000 (if using lossy codec)"))
    CONFIGPARAMETER  (m_fileName,  __("Filename prefix")                      )
    CONFIGINPUTPIN   (m_inputImage,__("Input image")                          )
    CONFIGCHOICEPARAM(m_imageType ,__("Image type"),m_imageTypeDescriptions,m_imageTypes)
END_CONFIG()


/***************************************************************************/
/*!
 *  Initializes members
 *
 ***************************************************************************/
imageSequenceWriter::imageSequenceWriter ():
vistaLab::module(OUTPUT,__("Output"),__("Image Sequence Writer"),
__("Writes Images Consecutively"),"Fabian Wenzel","wenzel@tu-harburg.de"),
m_quality(1000),
m_fileName(_("output_")),
m_imageType(PNG)
{
    m_imageTypes.clear();
    m_imageTypeDescriptions.clear();
    m_imageTypeSuffixes.clear();

    m_imageTypeDescriptions.push_back("PNG: Portable Network Graphic");
    m_imageTypes           .push_back(PNG);
    m_imageTypeSuffixes    .push_back("png");

    m_imageTypeDescriptions.push_back("JPG: JPEG");
    m_imageTypes           .push_back(JPG);
    m_imageTypeSuffixes    .push_back("jpg");

    m_imageTypeDescriptions.push_back("BMP: Windows Bitmap");
    m_imageTypes           .push_back(BMP);
    m_imageTypeSuffixes    .push_back("bmp");
}


/***************************************************************************/
/*!
 *  Destroys dynamic members
 *
 ***************************************************************************/
imageSequenceWriter::~imageSequenceWriter ()
{
}


/***************************************************************************/
/*!
 *  Initializes "runtime" variables which have to be reset at each run
 *
 ***************************************************************************/
bool imageSequenceWriter::init ()
{
    m_frameCounter=0;
    return true;
}


/***************************************************************************/
/*!
 *  Destroys dynamic "runtime" variables
 *
 ***************************************************************************/
void imageSequenceWriter::done ()
{
}


/***************************************************************************/
/*!
 *  Processes the data
 *
 ***************************************************************************/
bool imageSequenceWriter::process()
{
    char fileName[255];
    snprintf(fileName,255,"%s%03d.%s",m_fileName.c_str(),m_frameCounter++,
        m_imageTypeSuffixes[m_imageType].c_str());

    wxImage saveimage(m_inputImage->width(), m_inputImage->height(),
                        (unsigned char *) (&(*m_inputImage->begin())), true);
    saveimage.SaveFile(fileName);

    return true;
}
