/***************************************************************************
                   inputstreamimage.cpp  -  description
                            -------------------
    begin                : Fri Nov 16 2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "inputstreamimage.h"

#include <vigra/stdimage.hxx>

#include <wx/image.h>

using namespace std;

////////////////////////////// Public Functions /////////////////////////////

BEGIN_CONFIG(inputStreamImage,vistaLab::inputStreamFile)
    CONFIGPARAMETER(m_showOnce   ,__("Show Image Once"))
    CONFIGOUTPUTPIN(m_outputImage,__("Output Image"   ))
END_CONFIG()

/***************************************************************************/
/*!
 *  Initializes the list of provided colorspaced with GRAY and RGB
 *
 ***************************************************************************/
inputStreamImage::inputStreamImage() :
inputStreamFile(__("Image"),__("Loads a single Image from file"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_showOnce(false)
{
}


/***************************************************************************/
/*!
 *  If the end of the stream has not been hit ( which will be only when
 *  this function is called for the first time - see
 *  a new image is being read.
 *
 ***************************************************************************/
bool inputStreamImage::process()
{
    if (m_endOfStream)
    {
        vistaLab::setError(_("No more images"));
        return false;
    }

    wxImage tmp;
    if(!tmp.LoadFile(m_fileName.c_str()))
    {
        vistaLab::setError(_("Could not load image."));
        return false;
    }

    m_outputImage = new vigra::BRGBImage(tmp.GetWidth(),tmp.GetHeight(),
                        (vigra::RGBValue< unsigned char > *) tmp.GetData());

    if(m_showOnce)
        m_endOfStream = true;

    return true;
}
