/***************************************************************************
                inputstreamimagesequence.h  -  description
                            -------------------
    begin                : Mon Feb 18 2002
    copyright            : (C) 2001 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _INPUTSTREAMIMAGESEQUENCE_H_
#define _INPUTSTREAMIMAGESEQUENCE_H_

#include "vlmodule.h"

#include <vigra/stdimage.hxx>
#include <vector>
#include <string>


/***************************************************************************/
//! An input stream which reads a sequence of files
/*!
 *  After all files have been read sequentially, three options are possible:
 *
 *  - The input stream stops
 *  - The input streams starts over
 *  - The input streams loops backwards
 *
 * \ingroup Modules
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class inputStreamImageSequence : public vistaLab::module {
    VISTALABMODULE(inputStreamImageSequence)
public:
    inputStreamImageSequence();
    virtual bool process();
    virtual bool init();
    virtual void done();
private:

    //Flags indicating the behavior after the last image has been read
    PARAMETER(bool                       , m_loop            )
    PARAMETER(bool                       , m_loopBackAndForth)
    OUTPUTPIN(vigra::BRGBImage           , m_outputImage     )
    PARAMETER(vistaLab::multipleFileNames, m_fileNames       )

    bool m_currentlyLoopingForward;
    bool m_endOfStream;

    //Members for fileNames
    std::vector< std::string >::iterator m_currentFile;
    const char *checkFileNameSize();
};

#endif // #ifndef _INPUTSTREAMIMAGESEQUENCE_H_
