/***************************************************************************
                     integerscaler.cpp  -  description
                            -------------------
    begin                : Wed, Jul 17 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de
 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "integerscaler.h"
#include <vector>

using namespace std;
using namespace vigra;


///////////////////////////// Public Functions ///////////////////////////////

BEGIN_CONFIG(integerScaler,vistaLab::module)
    CONFIGINPUTPIN (m_inputImage,  __("Input image"           ))
    CONFIGOUTPUTPIN(m_outputImage, __("Output image"          ))
    CONFIGPARAMETER(m_factor,      __("Integer Scaling Factor (Negative values: shrink image)"))
END_CONFIG()

/***************************************************************************/
/*!
 *  
 *
 ***************************************************************************/
integerScaler::integerScaler ():
vistaLab::module (PROCESSING,__("Geometry"),__("Integer Scaler"),
__("Scales images integer times"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_factor(1)
{
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
bool integerScaler::init ()
{
    return true;
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
void integerScaler::done ()
{
}


/***************************************************************************/
/*!
 *   Actually scales the image
 *
 ***************************************************************************/
bool integerScaler::process()
{
    if(m_factor > 0){
        m_outputImage = new BRGBImage(m_inputImage->width()*m_factor,
            m_inputImage->height()*m_factor);

        BRGBImage::const_traverser src = m_inputImage ->upperLeft();
        BRGBImage::traverser       dst = m_outputImage->upperLeft();

        for(int sy=0;sy<m_inputImage->height();++sy,++src.y){
            BRGBImage::const_traverser srcrow(src);
            for(int dy=0;dy<m_factor;++dy,++dst.y)
            {
                BRGBImage::traverser dstrow(dst);
                for(int sx=0;sx<m_inputImage->width();++sx,++srcrow.x)
                    for(int dx=0;dx<m_factor;++dx,++dstrow.x)
                    {
                        *dstrow=*srcrow;
                    }
            }
        }
    } else {
        int absfactor = abs(m_factor);
        m_outputImage = new BRGBImage(m_inputImage->width()/absfactor,
            m_inputImage->height()/absfactor);

        BRGBImage::const_traverser src = m_inputImage ->upperLeft();
        BRGBImage::traverser       dst = m_outputImage->upperLeft();

        for(int dy=0;dy<m_outputImage->height();++dy,++dst.y){
            BRGBImage::traverser dstrow(dst);
            for(int sy=0;sy<absfactor;++sy,++src.y)
            {
                BRGBImage::const_traverser srcrow(src);
                for(int dx=0;dx<m_outputImage->width();++dx,++dstrow.x){
                    NumericTraits< BRGBImage::value_type >::Promote average;
                    for(int sx=0;sx<absfactor;++sx,++srcrow.x)
                    {
                        average+=*srcrow;
                    }
                    *dstrow=(average/absfactor/absfactor);
                }
            }
        }
    }
    return true;
}
