/***************************************************************************
                       logfile.cpp  -  description
                            -------------------
    begin                : Tue May 27 2003
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "logfile.h"

#include "vlpin.h"
#include "vlconfigmanager.h"
#include "vlapp.h"
#include "vlframe.h"

using namespace std;


//////////////////////////// Public Functions /////////////////////////////


BEGIN_CONFIG(logFile,vistaLab::module)
    CONFIGPARAMETER(m_separateFile, __("One file for each frame")                      )
    CONFIGPARAMETER(m_annotations , __("Annotations like frame number and module name"))
    CONFIGPARAMETER(m_fileName    , __("Log file name - without suffix")               )

    getConfigItems()->push_back(vistaLab::configItem(vistaLab::configItem::LOGINPUTPIN,
        (void *) &m_logSlots,__("Log Hook")));
END_CONFIG()


/***************************************************************************/
/*!
 *  Constructor
 *
 ***************************************************************************/
logFile::logFile() :
vistaLab::module(GUI,__("Output"),_("Log File"),
__("Writes log messages to a file"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_logFile(0),
m_separateFile(true),
m_annotations(true),
m_fileName("")
{
}


/***************************************************************************/
/*!
 *  Deletes the associated channel window
 *
 ***************************************************************************/
logFile::~logFile()
{
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
bool logFile::init()
{
    m_frameCounter = 0;
    if(!m_separateFile){
        snprintf(m_fullFileName,255,"%s.log",m_fileName.c_str());
        m_logFile = new fstream(m_fullFileName,ios::out);
    }
    return true;
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
void logFile::done()
{
    if(!m_separateFile && m_logFile){
        m_logFile->close();
        delete m_logFile;
        m_logFile = 0;
    }
}


/***************************************************************************/
/*!
 *  Hands the output image to the channelwindow
 *
 ***************************************************************************/
bool logFile::process()
{
    if (!m_logSlots.size())
        return true;

    if (m_separateFile){
        snprintf(m_fullFileName,255,"%s%03d.log",m_fileName.c_str(),
            currentFrameNumber());
        m_logFile = new fstream(m_fullFileName,ios::out);
    }

    if(m_annotations){
        *m_logFile << wxString::Format(_("***** Frame %d *****"),
            currentFrameNumber()) << endl << endl;
    }

    vector< int >::iterator currentLogSlot;
    for(currentLogSlot  = m_logSlots.begin();
        currentLogSlot != m_logSlots.end();
        ++currentLogSlot){
            // Call each renderpin
            vistaLab::logPin *currentLogPin =
                vistaLab::configManager::getManager().getLogPins()[*currentLogSlot];

            // Only process if this is an active
            // module (otherwise there could be missing data)
            if(currentLogPin->m_moduleInstance->isActive() && currentLogPin->m_moduleInstance->isValid()){

                string logString = ((currentLogPin->m_moduleInstance)->*
                    (currentLogPin->m_logFunction))();
                if (m_annotations)
                    *m_logFile << wxString::Format("[ %s ]:\n",
                    currentLogPin->m_moduleInstance->getInstanceName().c_str()) << endl;

                *m_logFile << logString.c_str();

                if(m_separateFile){
                    m_logFile->close();
                    delete m_logFile;
                } else if (m_annotations)
                    *m_logFile << endl;
            }
        }
        return true;
}
