/***************************************************************************
                     logwindow.cpp  -  description
                            -------------------
    begin                : Thu Jul 18 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/notebook.h>

#include "logwindow.h"

#include "vlpin.h"
#include "vlapp.h"
#include "vlconfigmanager.h"
#include "vlframe.h"

#include <algorithm>

using namespace std;

enum {
    LOG_SAVE = 0,
    LOG_CLEAR
};


/////////////////////////// Classes for log window ////////////////////////////



BEGIN_EVENT_TABLE(logWindowDialog,wxDialog)
    EVT_BUTTON(LOG_SAVE , logWindowDialog::onSave )
    EVT_BUTTON(LOG_CLEAR, logWindowDialog::onClear)
END_EVENT_TABLE()


/***************************************************************************/
/*!
 *  Constructor for log window
 *
 ***************************************************************************/
logWindowDialog::logWindowDialog(wxWindow *parent, int id,const wxString &title,
    const wxPoint &position, const wxSize &size, int style, logWindow *logModule):
wxDialog(parent,id,title,position,size,style),
p_logModule(logModule)
{
    m_topSizer = new wxBoxSizer(wxVERTICAL);

    m_notebook = new wxNotebook( this, -1, wxDefaultPosition,
                                 wxDefaultSize, wxNB_MULTILINE );

    m_topSizer->Add(m_notebook,wxSizerFlags(1).Expand().Border(wxALL,4));
    SetSizer(m_topSizer);

    wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
    buttonSizer->Add(new wxButton(this,LOG_SAVE,_("Save to Logfile...")),
                        wxSizerFlags(0).Expand().Border(wxALL,10));

    buttonSizer->Add(new wxButton(this,LOG_CLEAR,_("Clear Window")),
                        wxSizerFlags(0).Expand().Border(wxALL,10));

    m_topSizer->Add(buttonSizer,wxSizerFlags(0).Center());
}


/***************************************************************************/
/*!
 *  Saves current log textCtrl to file
 *
 ***************************************************************************/
void logWindowDialog::onSave(wxCommandEvent &event)
{
    wxString defaultFileName;

    if(m_notebook->GetSelection() != -1){
        defaultFileName =
            m_notebook->GetPageText(m_notebook->GetSelection())+
            wxString::Format("_frame_%d",vistaLab::module::currentFrameNumber())+".log";

        wxString fileName =  wxFileSelector(_("Save Log file"),
            wxEmptyString,defaultFileName,"*.log",
            _("Log Files|*.log|All files|*.*"),
            wxSAVE|wxOVERWRITE_PROMPT);

        if(fileName != wxEmptyString){
            if (wxFileExists(fileName))
                wxRemoveFile(fileName);

            list< logWindow::SLogPage >::iterator page =
                p_logModule->getPage(m_notebook->GetPageText(m_notebook->GetSelection()));

            page->textCtrl->SaveFile(fileName);
        }
    }
}


/***************************************************************************/
/*!
 *  Clears current textCtrl in log window
 *
 ***************************************************************************/
void logWindowDialog::onClear(wxCommandEvent &event)
{
    if(m_notebook->GetSelection() != -1){
        list< logWindow::SLogPage >::iterator page =
            p_logModule->getPage(m_notebook->GetPageText(m_notebook->GetSelection()));
        page->textCtrl->Clear();
    }
}


wxColour logWindow::m_instanceColor;
wxColour logWindow::m_frameColor;


//////////////////////////// Public Functions /////////////////////////////


BEGIN_CONFIG(logWindow,vistaLab::module)
    CONFIGPARAMETER(m_historyLength, __("Length of history for each module in characters"))
    CONFIGPARAMETER(m_clearPage,     __("Clear log window after each frame")              )
    CONFIGPARAMETER(m_rawOutput,     __("Raw output (suppress frame nr. etc)")            )

    getConfigItems()->push_back(vistaLab::configItem(
        vistaLab::configItem::LOGINPUTPIN, (void *) &m_logSlots,__("Log Hook")));
END_CONFIG()


/***************************************************************************/
/*!
 *  Registers an RGB input image. The instance colour has to be initialized
 *  here as there is no wxTheColourDatabase in static initialization
 *
 ***************************************************************************/
logWindow::logWindow(): vistaLab::module(GUI,__("Output"),__("Log Window"),
__("Displays log messages from other modules"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_historyLength(16000),
m_clearPage(false),
m_rawOutput(false),
m_logWindow(0)
{
    m_instanceColor=wxTheColourDatabase->Find("FOREST GREEN");
    m_frameColor   =wxTheColourDatabase->Find("INDIAN RED");
}


/***************************************************************************/
/*!
 *  Deletes the associated log window
 *
 ***************************************************************************/
logWindow::~logWindow()
{
    if(m_logWindow)
        m_logWindow->Destroy();
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
bool logWindow::init()
{
    if(!m_logWindow){
        m_logWindow = new logWindowDialog(NULL,-1,
            wxString(getInstanceName().c_str()),
            wxDefaultPosition,wxSize(500,500),
            wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER,this);
    }

    updatePages();

    if(!wxGetApp().singleRun())
        m_logWindow->Show(true);

    return true;
}


/***************************************************************************/
/*!
 *  Does nothing
 *
 ***************************************************************************/
void logWindow::done()
{
}


/***************************************************************************/
/*!
 *  Hands the output image to the channelwindow
 *
 ***************************************************************************/
bool logWindow::process()
{
    vector< int >::iterator currentLogSlot;
    for(currentLogSlot  = m_logSlots.begin();
        currentLogSlot != m_logSlots.end();
        ++currentLogSlot){

        // Call each renderpin
        vistaLab::logPin *currentLogPin =
            vistaLab::configManager::getManager().getLogPins()[*currentLogSlot];

        // Only process if this is an active
        // module (otherwise there could be missing data)
        if(currentLogPin->m_moduleInstance->isActive() &&
           currentLogPin->m_moduleInstance->isValid()){

            list< SLogPage >::iterator page =
                getPage(currentLogPin->m_moduleInstance->getInstanceName().c_str(),&m_logWidgets);

            if(m_clearPage)
                page->textCtrl->Clear();
            else if(!m_rawOutput){
                page->textCtrl->SetForegroundColour(m_frameColor);
                page->textCtrl->AppendText(wxString::Format(_("\n***** Frame %d *****\n\n"),
                    currentFrameNumber()));
                page->textCtrl->SetForegroundColour(*wxBLACK);
            }

            string logstring = ((currentLogPin->m_moduleInstance)->*
                (currentLogPin->m_logFunction))();


            page->textCtrl->AppendText(logstring.c_str());

            if(!m_rawOutput)
                page->textCtrl->AppendText("\n");

            int numberOfChars = page->textCtrl->GetLastPosition();
            if(numberOfChars > m_historyLength)
                page->textCtrl->Remove(0,numberOfChars-m_historyLength);
        }
    }
    return true;
}


/***************************************************************************/
/*!
 *  Changes the window title
 *
 ***************************************************************************/
void logWindow::postConfig()
{
    if(m_logWindow)
        m_logWindow->SetTitle(getInstanceName().c_str());
}


/***************************************************************************/
/*!
 *  Updates Notebook pages
 *
 ***************************************************************************/
void logWindow::updatePages()
{

    list< SLogPage > copyOfLogwidgets(m_logWidgets);
    vector< int >::iterator currentLogSlot;

    for(currentLogSlot  = m_logSlots.begin();
        currentLogSlot != m_logSlots.end();
        ++currentLogSlot){

            vistaLab::logPin *currentLogPin = vistaLab::configManager::getManager().getLogPins()[*currentLogSlot];
            wxString instanceName = currentLogPin->m_moduleInstance->getInstanceName().c_str();
            list< SLogPage >::iterator match = getPage(instanceName,&copyOfLogwidgets);
            if(match == copyOfLogwidgets.end()){
                // New page
                SLogPage newPage;
                wxPanel *panel = new wxPanel(m_logWindow->GetNotebook());

                wxBoxSizer *pageSizer = new wxBoxSizer(wxVERTICAL);
                panel->SetSizer(pageSizer);

                newPage.textCtrl = new wxTextCtrl(panel,wxID_ANY,wxEmptyString,
                    wxDefaultPosition,wxSize(420,380),
                    wxTE_MULTILINE);
                pageSizer->Add(newPage.textCtrl,wxSizerFlags(1).Expand());

                newPage.title = instanceName;

                m_logWindow->GetNotebook()->AddPage( panel, instanceName);
                m_logWidgets.push_back(newPage);
            } else {
                match->textCtrl->Clear();
                copyOfLogwidgets.erase(match);
            }

        }

        // Delete remaining pages

        list< SLogPage >::iterator remainingPage = copyOfLogwidgets.begin();

        while(remainingPage != copyOfLogwidgets.end()){
            list< SLogPage >::iterator pageToBeDeleted =
                find(m_logWidgets.begin(),m_logWidgets.end(),*remainingPage);

            for(unsigned int i=0;i<m_logWindow->GetNotebook()->GetPageCount();++i)
                if( m_logWindow->GetNotebook()->GetPageText(i) == pageToBeDeleted->title )
                    m_logWindow->GetNotebook()->DeletePage(i);

            m_logWidgets.erase(pageToBeDeleted);
            ++remainingPage;
        }
    m_logWindow->getTopSizer()->SetSizeHints(m_logWindow);
    m_logWindow->getTopSizer()->Fit(m_logWindow);
}


/***************************************************************************/
/*!
 *  Gets Page given name
 *
 ***************************************************************************/
list< logWindow::SLogPage >::iterator
logWindow::getPage(const wxString &title, list< logWindow::SLogPage > *pages)
{
    if(pages == 0)
        pages = &m_logWidgets;

    list< SLogPage >::iterator matchingPage = pages->begin();

    while (matchingPage != pages->end() && matchingPage->title != title)
        ++matchingPage;

    return matchingPage;
}
