/***************************************************************************
                       logwindow.h  -  description
                            -------------------
    begin                : Thu Aug 08 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _LOGWINDOW_H_
#define _LOGWINDOW_H_

#include "vlmodule.h"
#include <wx/dialog.h>

class wxTextCtrl;
class wxColour;
class wxPanel;
class wxNotebook;
class logWindow;

class logWindowDialog : public wxDialog {
public:
    logWindowDialog(wxWindow *parent, int id=-1,
        const wxString &title=wxEmptyString, const wxPoint &position = wxDefaultPosition,
        const wxSize &size = wxDefaultSize, int style=0,logWindow *logmodule=0);
    void onSave(wxCommandEvent &event);
    void onClear(wxCommandEvent &event);
    wxNotebook *GetNotebook()
    {
        return m_notebook;
    }

    wxBoxSizer *getTopSizer()
    {
        return m_topSizer;
    }

private:
    DECLARE_EVENT_TABLE()
    wxNotebook *m_notebook;
    logWindow *p_logModule;
    wxBoxSizer *m_topSizer;
};

/***************************************************************************/
//! Implements a log window which calls displays the output of log functions
/*!
 * \ingroup Modules
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class logWindow : public vistaLab::module {
    VISTALABMODULE(logWindow);
public:
    logWindow();
    virtual ~logWindow();
    virtual bool init();
    virtual void done();
    virtual bool process();
    virtual void postConfig();

    struct SLogPage {
        wxTextCtrl *textCtrl;
        wxString    title;

        bool operator==(const SLogPage &other)
        {
            return textCtrl == other.textCtrl;
        }
    };

    std::list< SLogPage >::iterator getPage(const wxString &title, std::list< SLogPage > *pages=0);

private:
    std::vector< int > m_logSlots;
    void updatePages();

    PARAMETER(int,  m_historyLength);
    PARAMETER(bool, m_clearPage    );
    PARAMETER(bool, m_rawOutput    );

    static wxColour m_instanceColor;
    static wxColour m_frameColor;
    std::list< SLogPage > m_logWidgets;
    logWindowDialog   *m_logWindow;
};

#endif // #ifndef _LOGWINDOW_H_
