/***************************************************************************
                       rotator.cpp  -  description
                            -------------------
    begin                : Mon Jun 3 2002
    copyright            : (C) 2002 by wenzel
    email                : wenzel@tu-harburg.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "rotator.h"

#include <vigra/stdimage.hxx>
#include <vigra/basicgeometry.hxx>

using namespace vigra;
using namespace std;


BEGIN_CONFIG(rotator,vistaLab::module)
    CONFIGINPUTPIN  (m_inputImage,  __("Input Image"))
    CONFIGOUTPUTPIN (m_rotatedImage,__("Output Image"))
    CONFIGPARAMETER (m_numOf90DegCCW, __("Rotate ... times 90 degrees CCWwise"))
END_CONFIG()

//////////////////////////// Public Functions /////////////////////////////


/***************************************************************************/
/*!
 *  Constructor sets up default values
 *
 ***************************************************************************/
rotator::rotator() :
vistaLab::module(PROCESSING,__("Geometry"),__("Image Rotator"),
__("Rotates the input image"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_numOf90DegCCW(1)
{
}


/***************************************************************************/
/*!
 *  No initialization necessary
 *
 ***************************************************************************/
bool rotator::init()
{
    return true;
}


/***************************************************************************/
/*!
 *  No cleaning up necessary
 *
 ***************************************************************************/
void rotator::done()
{
}


/***************************************************************************/
/*!
 *  Does the rotation
 *
 ***************************************************************************/
bool rotator::process()
{

    // Determine new image extents
    int newWidth, newHeight;
    if ( m_numOf90DegCCW & 1 ){
        newWidth = m_inputImage->height();
        newHeight= m_inputImage->width();
    } else {
        newWidth = m_inputImage->width();
        newHeight= m_inputImage->height();
    }

    // Initialize rotated image
    m_rotatedImage = new BRGBImage(newWidth,newHeight);

    rotateImage(srcImageRange(*m_inputImage),destImage(*m_rotatedImage),m_numOf90DegCCW * 90);

    return true;
}
