/***************************************************************************
                     sobelfilter.cpp  -  description
                            -------------------
    begin                : Tue Feb 19 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "sobelfilter.h"

#include <vigra/stdconvolution.hxx>
#include <vigra/functorexpression.hxx>
#include <vigra/combineimages.hxx>

using namespace vigra;
using namespace std;



//////////////////////////// Public Functions /////////////////////////////

BEGIN_CONFIG(sobelFilter,vistaLab::module)
    CONFIGINPUTPIN (m_inputImage, __("Input Image"))
    CONFIGOUTPUTPIN(m_outputImage,__("Output Image"))
END_CONFIG()

/***************************************************************************/
/*!
 *  Constructor sets up default values
 *
 ***************************************************************************/
sobelFilter::sobelFilter() :
vistaLab::module(PROCESSING,__("Filter"),__("Sobel"),
__("Filters a gray image"),
"Fabian Wenzel","wenzel@tu-harburg.de")
{
}


/***************************************************************************/
/*!
 *  There is no initialization
 *
 ***************************************************************************/
bool sobelFilter::init()
{
    return true;
}


/***************************************************************************/
/*!
 *  There is no cleaning up
 *
 ***************************************************************************/
void sobelFilter::done()
{
}


/***************************************************************************/
/*!
 *
 *
 ***************************************************************************/
bool sobelFilter::process()
{
    m_outputImage = new BImage(m_inputImage->size());

    BImage::const_traverser src = m_inputImage ->upperLeft()+Diff2D(1,1);
    BImage::traverser       dst = m_outputImage->upperLeft()+Diff2D(1,1);

    for(int y=1;y<m_inputImage->height()-1;++y,++src.y,++dst.y)
    {
        BImage::const_traverser srcrow(src);
        BImage::traverser       dstrow(dst);
        for(int x=1;x<m_inputImage->width()-1;++x,++src.x,++dst.x)
        {
            //Result of horizontal sobel
            NumericTraits< BImage::value_type >::Promote
            tmp_x =     srcrow[Diff2D(1,-1)]-srcrow[Diff2D(-1,-1)]
                    +   srcrow[Diff2D(1, 1)]-srcrow[Diff2D(-1, 1)]
                    +2*(srcrow[Diff2D(1, 0)]-srcrow[Diff2D(-1, 0)]);

            NumericTraits< BImage::value_type >::Promote
            tmp_y =     srcrow[Diff2D(-1,1)]-srcrow[Diff2D(-1,-1)]
                    +   srcrow[Diff2D( 1,1)]-srcrow[Diff2D( 1,-1)]
                    +2*(srcrow[Diff2D( 0,1)]-srcrow[Diff2D( 0,-1)]);

            *dstrow = (BImage::value_type) sqrt((float) tmp_x*tmp_x+tmp_y*tmp_y);
        }
    }

    return true;
}
