/***************************************************************************
                     thresholder.cpp  -  description
                            -------------------
    begin                : Fri Aug 30 2002
    copyright            : (C) 2002 by wenzel
    email                : wenzel@tu-harburg.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#include "thresholder.h"

#include<vigra/transformimage.hxx>

using namespace std;


//////////////////////////// Public Functions /////////////////////////////

BEGIN_CONFIG(thresholder,vistaLab::module)
    CONFIGINPUTPIN (m_inputImage      ,__("Input Image"))
    CONFIGOUTPUTPIN(m_thresholdedImage,__("Binarized Image"))
    CONFIGPARAMETER(m_threshold       ,__("Intensity Threshold"))
    CONFIGPARAMETER(m_lowValue        ,__("Replacement for Low Intensities"))
    CONFIGPARAMETER(m_highValue       ,__("Replacement for High Intensities"))
    CONFIGVAL      (checkValues)
END_CONFIG()


/***************************************************************************/
/*!
 *  Constructor sets up default values
 *
 ***************************************************************************/
thresholder::thresholder() :
vistaLab::module(PROCESSING,__("Filter"),__("Image Threshold"),
__("Applies a threshold to a gray image"),
"Fabian Wenzel","wenzel@tu-harburg.de"),
m_threshold(128),
m_lowValue(0),
m_highValue(255)
{
}


/***************************************************************************/
/*!
 *  No initialization necessary
 *
 ***************************************************************************/
bool thresholder::init()
{
    return true;
}


/***************************************************************************/
/*!
 *  No cleaning up necessary
 *
 ***************************************************************************/
void thresholder::done()
{
}


/***************************************************************************/
/*!
 *  Applies the threshold to the image
 *
 ***************************************************************************/
bool thresholder::process()
{
    m_thresholdedImage = new vigra::BImage(m_inputImage->size());

    transformImage(srcImageRange(*m_inputImage),destImage(*m_thresholdedImage),
         vigra::Threshold< vigra::BImage::PixelType, vigra::BImage::PixelType>
            (0, m_threshold, m_lowValue, m_highValue));

    return true;
}


/***************************************************************************/
/*!
 *  Checks if values are in valid range
 *
 ***************************************************************************/
const char *thresholder::checkValues()
{
    if(m_threshold<0 || m_threshold > 255)
        return _("The threshold must be between 0 and 255");

    if(m_lowValue<0 || m_lowValue > 255)
        return _("The Replacement for Low Intensities must be between 0 and 255");

    if(m_highValue<0 || m_highValue > 255)
        return _("The Replacement for High Intensities must be between 0 and 255");

    return NULL;
}
