/***************************************************************************
                            vlapp.h  -  description
                             -------------------
        begin                : Fri Nov 16 2001
        copyright            : (C) 2001 by Fabian Wenzel
        email                : wenzel@tu-harburg.de

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLAPP_H_
#define _VLAPP_H_

#include <wx/app.h>
#include <wx/intl.h>
#include <wx/string.h>

namespace vistaLab {

class processor;
class repository;
class document;

#ifdef __WXDEBUG__
#define VLAPPNAME _("VistaLab (Debug Version)")
#else
#define VLAPPNAME _("VistaLab")
#endif

/***************************************************************************/
//! The application object
/*!
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class app : public wxApp  {
public:

  // Overwritten virtual functions
  virtual bool OnInit();
  virtual int  OnExit();

  // Return "global" members as document, processor or repository
  processor  *getProcessor () { return m_processor;  }
  repository *getRepository() { return m_repository; }
  document   *getDocument  () { return m_document;   }

  // Return plugin path (for plugins)
  const wxString   getPluginDir ();
  const wxString   getLocaleDir ();

  void addLocaleCatalog(const wxString &catalogName);

  // Creates main window
  void createMainWindow();

  // Flags for starting up
  bool singleRun();

private:
  // The full program name which is used for figuring out
  // whether installed or uninstalled modules should be loaded
  wxString m_fullProgramName;

  // The members of the app
  repository *m_repository;
  processor  *m_processor;
  document   *m_document;

  // The locale has to be a member as other catalogs might be added later
  wxLocale         m_locale;
  bool             m_singleRun;
  bool             m_showMainWindow;
  wxString         m_bootProject;

  bool parseOptions();
  bool showSplashScreen();
  const wxString &bootProject() const;
  void showOptions() const;
  bool showMainWindow() const;
};

} // namespace vistaLab

DECLARE_APP(vistaLab::app)

#endif // #ifndef _VLAPP_H_
