/***************************************************************************
                        vlchannelWindow.h  -  description
                            -------------------
    begin                : Wed Jan 2 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLCHANNELWINDOW_H_
#define _VLCHANNELWINDOW_H_

#include <wx/panel.h>
#include <vigra/stdimage.hxx>

class imageChannel;
class inputThread;

class wxBitmap;
class wxBoxSizer;
class wxStaticBox;
class wxStaticBoxSizer;
class wxStaticText;
class wxBitmap;

namespace vistaLab {

class channelWindow;
class view;

////////////////////// Functions for the channel target //////////////////////


/***************************************************************************/
//! The channel target - this is the window which shows the target image
/*!
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class channelTarget : public wxPanel {
public:
    channelTarget(wxWindow *parent);
    ~channelTarget();

    // Interface for channelWindow
    void refreshFrame(const vigra::BRGBImage &targetImage);
    void clear();
    void updateSize  (int width, int height);
private:
    // The bitmap which stores the target image internally
    wxBitmap             *m_targetBitmap;
    wxCriticalSection     m_bitmapLock;
    // Callback functions
    void onPaint          (wxPaintEvent &event);
    void onMouse          (wxMouseEvent &e);
    void onMouseMotion    (wxMouseEvent &e);
    void onEraseBackground(wxEraseEvent &e);
    int  m_x,m_y;
    bool m_rightMouseDown;

    DECLARE_EVENT_TABLE();
};




////////////////////// Functions for the channel window //////////////////////


/***************************************************************************/
//! The window. This keeps the channel target as well some decorations
/*!
 *  The decorations are a box around it, the channel instance name
 *  and currently a test string
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class channelWindow : public wxPanel {
friend class channelTarget;
public:
    channelWindow(wxWindow *parent, imageChannel *module);
    ~channelWindow();


    // Update functions (might be combined?!)
    void eatImage(const vigra::BRGBImage &displayImage);

    void ToggleShowPixelValues(bool show);

    void showPixelValues (int x, int y);
    void clearPixelValues();

    void updateInstanceName();

private:
    // Callbacks
    void onMouse         (wxMouseEvent   &event);
    void onSnapshot      (wxCommandEvent &event);

    // Indicator whether the right mouse button is up or down
    bool m_rightMouseDown;

    // The render target which keeps the current target image
    channelTarget *m_renderTarget;

    // The connected module (for getting updated instance names)
    imageChannel *m_connectedModule;

    // GUI controls/helpers
    wxBoxSizer              *m_topSizer;
    wxStaticText            *m_legend;
    view                    *p_view;
    vigra::BRGBImage         m_targetImage;

    // Information for the displayed frame and a pointer to it
    DECLARE_EVENT_TABLE()
};

} // namespace vistaLab

#endif // #ifndef _VLCHANNELWINDOW_H_
