/***************************************************************************
                      vlconfigmanager.h  -  description
                            -------------------
    begin                : Mit Mai 22 2002
    copyright            : (C) 2002 by wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLCONFIGMANAGER_H_
#define _VLCONFIGMANAGER_H_

#include <vector>
#include <string>
#include <map>
#include "vlconfigitem.h"
#include "vlmodule.h"
#include "vlprocessor.h"

namespace vistaLab {

class outputPin;
class renderPin;
class logPin;


/***************************************************************************/
//! Singleton which assigns IDs to all data types that processors produce
/*!
 *
 * If a processor produces data in CProcessor::process() it should usually
 * be attached to the frame such that
 *
 * - the same processor can display it later
 * - other processors may use the data for further computations
 *
 * Referencing data elements in a frame is done with IDs which are created
 * by CConfigManager. Each processor has to register its data types
 * with a call to registerDataType. This function returns an ID which then
 * should be used as an argument in CFrame::addAgentData() or
 * CFrame::getAgentData(). Finally, if another processor needs a special
 * datatype, it may query for a certain datatype by calling
 * addConfigProcessorData() which will return the corresponding ID when
 * configuring. This may be used for accessing the same way.
 *
 * In order not to link this class to the whole framework it is implemented
 * as a singleton.
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class configManager {
public:
    typedef std::vector< outputPin *>              TOutputPins;
    typedef std::vector< renderPin *>              TRenderPins;
    typedef std::vector< logPin    *>              TLogPins;
    typedef std::vector< module    *>              TActiveModules;

    //! Entry point for the singleton
    static configManager &getManager();

    void addActiveModule   (module *newModule);

    //! Sets active-flags in modules of active modules
    void setValidModules();

    //! Updates and reconnects all pins (when the module order has changed)
    void updatePins();

    //! Get processor names which all provide the queried datatype
    TOutputPins getMatchingOutputPins (const std::string &type,
        const configurableObject *module);

    outputPin *getOutputPin(const int slot);

    //! The maximum number of datatypes
    int getEntries()             { return m_outputPins.size(); }
    TRenderPins &getRenderPins() { return m_renderPins;        }
    TLogPins    &getLogPins()    { return m_logPins;           }
    ~configManager();

private:
    bool isValidModule(TActiveModules::iterator begin,
        TActiveModules::iterator currentModule);

    void clearPins(TOutputPins *outputPins, TRenderPins *renderPins,
        TLogPins *logPins);

    //! Datastructures for storing data type information
    TOutputPins m_outputPins;
    TRenderPins m_renderPins;
    TLogPins    m_logPins;

    //! Private constructors / destructors
    configManager();
};

} // namespace vistaLab

#endif // #ifndef _VLCONFIGMANAGER_H_
