/***************************************************************************
                        vldocument.h  -  description
                            -------------------
    begin                : Fri Nov 16 2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLDOCUMENT_H_
#define _VLDOCUMENT_H_

#include <wx/thread.h>

#include <vector>
#include "vlmodule.h"


namespace vistaLab {

class slice;


//////////////////////// Functions for the document ////////////////////////


/***************************************************************************/
//! Stores all information being created during runtime
/*!
 *  It handles incoming images and signals the process-core when
 *  processing can begin. The buffer which is used has one additional
 *  member compared with the m_bufferLength - variable. This is
 *  because one data spot has to be used for new input - thus is cannot
 *  be displayed or processed. 
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class document : public configurableObject {
    VISTALABTOPMODULE(document)
public:
    document();
    ~document();

    // Input of new images
    void addData( module *stream, slice *data );
    slice *getCurrentSlice() const { return *m_currentOutputSlice; }

    // Buffer handling
    int  getBufferLength() { return m_bufferLength; }
    bool getWaitFlag()     { return m_waitFlag; }
    void resetBuffers();
    void shiftSlicePointer(bool downwards);
    void shiftBuffer();
    void postConfig();

    double getFramerate() { return m_frameRate; }

private:

    // Internal members for new slices / buffer dimensions
    PARAMETER(int   ,m_bufferLength)
    PARAMETER(bool  ,m_waitFlag)
    PARAMETER(double,m_frameRate)

    bool m_sliceIsFull;

    // The buffer itself and its iterators
    std::vector<slice *>    m_buffer;
    slice                  *m_preBuffer;

    wxCriticalSection        m_preBufferLock;

    std::vector<slice *>::iterator m_currentInputSlice;
    std::vector<slice *>::iterator m_currentOutputSlice;
};

} // namespace vistaLab

#endif // #ifndef _VLDOCUMENT_H_
