/***************************************************************************
                        vlframe.cpp  -  description
                            -------------------
    begin                : Fri Nov 16 2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "vlframe.h"

#include "vlrepository.h"
#include "vlapp.h"
#include "vldocument.h"
#include "vlchannelwindow.h"
#include "vlframecontrols.h"
#include "vlview.h"
#include "vlconfigmanager.h"
#include "vlprocessor.h"
#include "vlmodule.h"
#include "vlinputthread.h"
#include "vlmoduleconfig.h"
#include "vlmoduleconfigfile.h"
#include "vlmoduleconfigdialog.h"
#include "vlprojectconfigwxfile.h"
#include "vlprojectconfigxml.h"

#include <vector>
#include <algorithm>

#include <wx/statline.h>
#include <wx/artprov.h>
#include <wx/filedlg.h>
#include <wx/listbox.h>
#include <wx/button.h>
#include <wx/slider.h>
#include <wx/menu.h>
#include <wx/msgdlg.h>
#include <wx/choicdlg.h>
#include <wx/statusbr.h>
#include <wx/sizer.h>
#include <wx/settings.h>
#include <wx/laywin.h>
#include <wx/filename.h>
#include <locale.h>

// Toolbar Icons
#include "resources/openproject.xpm"
#include "resources/newproject.xpm"
#include "resources/saveproject.xpm"
#include "resources/stop.xpm"
#include "resources/run.xpm"
#include "resources/pause.xpm"
#include "resources/configure.xpm"
#include "resources/testmodules.xpm"

// Main Icon
#ifndef WIN32
#include "resources/vistalab.xpm"
#endif

using namespace std;
using namespace vistaLab;


////////////////////////// Functions for the Frame //////////////////////////

const wxEventType vistaLab::wxEVT_VISTALAB = wxNewEventType();

#define EVT_VISTALAB(id, fn) \
    DECLARE_EVENT_TABLE_ENTRY( \
    wxEVT_VISTALAB, id, wxID_ANY, \
    (wxObjectEventFunction)(wxEventFunction) wxStaticCastEvent(wxCommandEventFunction,&fn), \
    (wxObject *) NULL \
    ),

/***************************************************************************
 *
 *  Event table
 *
 ***************************************************************************/
BEGIN_EVENT_TABLE(frame,wxFrame)
EVT_MENU  (VISTALABFILE_OPEN,    frame::projectOpen      )
EVT_MENU  (VISTALABFILE_SAVEAS,  frame::onProjectSaveAs  )
EVT_MENU  (VISTALABFILE_SAVE,    frame::onProjectSave    )
EVT_MENU  (VISTALABFILE_CLOSE,   frame::onProjectClose   )
EVT_MENU  (VISTALABFILE_RECENT,  frame::onRecentProject  )
EVT_MENU  (wxID_EXIT,         frame::onQuit           )
EVT_MENU  (HELP_ABOUT,        frame::onAbout          )
EVT_MENU  (PROCESS_START,     frame::processingStart  )
EVT_MENU  (PROCESS_PAUSE,     frame::processingPause  )
EVT_MENU  (PROCESS_STOP,      frame::processingStop   )
EVT_MENU  (EDIT_OPTIONS,      frame::globalOptions    )
EVT_MENU  (EDIT_REFRESH,      frame::onReloadModules  )
EVT_MENU  (SHOW_SPLASH,       frame::onShowSplash     )
EVT_MENU  (AUTORENAME,        frame::onAutoRename     )
EVT_MENU  (UNITTEST_MODULES,  frame::onUnitTest       )
EVT_SIZE  (                   frame::onSize           )
EVT_SASH_DRAGGED(ID_SASH,     frame::onSashDrag       )
EVT_CLOSE (                   frame::onClose          )
EVT_SLIDER(ID_FRAME_SLIDER,   frame::onSlider         )
EVT_VISTALAB(DISPLAY_ERROR,   frame::onDisplayError   )
EVT_VISTALAB(UPDATE_TICKS,    frame::onUpdateTicks    )
END_EVENT_TABLE()



/***************************************************************************/
/*!
 *  Constructor sets up its size and its child (the scrolled window
 *  vistaLabView) and sets up menus
 *
 ***************************************************************************/
frame::frame(wxWindow* parent, wxWindowID id,
                       const wxString& title) :
wxFrame              (parent, id, title,wxDefaultPosition,wxSize(1024,768)),
m_autoRename         (true),
m_lowestMenuID       (20000),
m_recentProjectMenuID(m_lowestMenuID+1000)
{

    // First, create repositorySashwindow and repositorytree on the left

    m_sash = new wxSashLayoutWindow(this,ID_SASH,
        wxDefaultPosition, wxSize(200, 800),
        wxNO_BORDER | wxSW_3D | wxCLIP_CHILDREN);
    m_sash->SetDefaultSize(wxSize(200, 800));
    m_sash->SetOrientation(wxLAYOUT_VERTICAL);
    m_sash->SetAlignment  (wxLAYOUT_LEFT);
    m_sash->SetSashVisible(wxSASH_RIGHT, true);


    m_notebook = new wxNotebook(this,-1);

    m_activeModuleList = new activeProjectTree(m_sash,ID_ACTIVEMODULELIST);

    wxPanel *repositoryPanel = new wxPanel(m_notebook,-1);
    wxSizer *repositorySizer = new wxBoxSizer(wxVERTICAL);

    m_repositoryTree = new repositoryTree(repositoryPanel,
        ID_REPOSITORYTREE);

    repositoryPanel->SetSizer(repositorySizer);

    repositorySizer->Add(m_repositoryTree,wxSizerFlags(1).Expand());

    repositorySizer->SetSizeHints(repositoryPanel);
    repositorySizer->Fit(repositoryPanel);

    // Now, create slider_view_panel which holds the output view
    // and the slider (both get their layout with sizers)
    m_sliderViewPanel = new wxPanel(m_notebook,-1);

    m_notebook->AddPage(m_sliderViewPanel,_("Project Output"));
    m_notebook->AddPage(repositoryPanel,_("Repository"));

    m_view   = new view(m_sliderViewPanel);
    m_slider = new wxSlider(m_sliderViewPanel,ID_FRAME_SLIDER,0,0,
        wxGetApp().getDocument()->getBufferLength()-1,
        wxDefaultPosition,wxDefaultSize,
        wxSL_LABELS| wxSL_HORIZONTAL | wxSL_AUTOTICKS);


    m_topSizer = new wxBoxSizer(wxVERTICAL);
    m_topSizer->Add(m_view,wxSizerFlags(1).Expand());
    m_topSizer->Add( new wxStaticLine( this, -1, wxDefaultPosition,
        wxSize(3,3), wxHORIZONTAL),wxSizerFlags(0).Expand().Border(wxALL,0));

#ifdef WXWIN_COMPATIBILITY_2_4
    m_topSizer->SetItemMinSize((unsigned int) 0,640,400);
#else
    m_topSizer->SetItemMinSize(0,640,400);
#endif
    m_topSizer->Add(m_slider,wxSizerFlags(0).Expand().Border(wxALL,5));

    m_sliderViewPanel->SetSizer(m_topSizer);
    m_topSizer->SetSizeHints(m_sliderViewPanel);
    m_topSizer->Layout();

    SetIcon(wxICON(vistalab));

    SetAutoLayout(true);

    // Create Status bar and set correct widths
    CreateStatusBar(4);
    int x,y;
    int widths[4];
    widths[0]=-1;
    GetTextExtent(_("Missed XXXX frames"),&x, &y);
    widths[1]=x;
    GetTextExtent(_("Frame XXXX"),       &x, &y);
    widths[2]=x;
    GetTextExtent(_("XXXX Modules"),     &x, &y);
    widths[3]= x;
    SetStatusWidths(4, widths);


#ifdef VERSION
    SetTitle(wxGetApp().GetAppName()+_(" - Version ")+wxString(VERSION));
#endif

    // Sets up menu and toolBar
    setupMenu();

    // Sets up recent files
    loadRecentProjects  ();
    updateRecentProjects();

    m_currentProject = wxEmptyString;
}


/***************************************************************************/
/*!
 *  Destructor saves the recent projects to the config-file for next time
 *
 ***************************************************************************/
frame::~frame()
{
    saveRecentProjects();
}


///////////////////// Private functions for the frame /////////////////////



/***************************************************************************/
/*!
 *  Closes the window
 *
 ***************************************************************************/
void frame::onQuit(wxCommandEvent &e)
{
    Close(true);
}


/***************************************************************************/
/*!
 *  Sets up the menus (including all dynamic items from the repository)
 *
 ***************************************************************************/
void frame::setupMenu()
{
    wxMenuBar *menuBar = new wxMenuBar(wxMB_DOCKABLE);

    // First menu: File manu
    wxMenu *file = new wxMenu();

    file->Append(VISTALABFILE_OPEN, _("Open...\tCtrl-O"),
        _("Opens a project file from disk"));
    file->Append(VISTALABFILE_SAVE, _("Save\tCtrl-S"),
        _("Saves a project file to disk"));  
    file->Append(VISTALABFILE_SAVEAS, _("Save as..."),
        _("Saves a project file to disk"));
    file->Append(VISTALABFILE_CLOSE,_("Close\tCtrl-W"),
        _("Closes a project"));
    file->AppendSeparator();
    // Recent projects will be inserted here
    file->AppendSeparator();
    file->Append(wxID_EXIT,_("&Quit\tAlt-F4"),_("Quits the application"));

    // Edit menu
    wxMenu *edit = new wxMenu();

    edit->Append(PROCESS_START,_("Start processing\tCtrl-R"),
        _("Starts processing of images"));
    edit->Append(PROCESS_STOP, _("Stop processing\tCtrl-C"),
        _("Stops processing of images"));

    edit->AppendSeparator();

    edit->Append(EDIT_REFRESH,_("Reload modules\tF5"));

    edit->AppendSeparator();

    edit->Append(UNITTEST_MODULES, _("Run Unit Tests"),
        _("Runs Unit Tests on all modules"));
    edit->AppendSeparator();

    edit->Append(EDIT_OPTIONS,_("Options...\tCtrl-P"),
        _("Sets global options"));
    edit->Append(SHOW_SPLASH,_("Show Splash Screen"),_("Toggles Splash Screen Behavior"),wxITEM_CHECK);
    edit->Append(AUTORENAME,_("Autorename Instances"),_("Rename identical Instance names automatically"),wxITEM_CHECK);

    bool tmp=true;
    wxConfig::Get()->Read("ShowSplash",&tmp);
    edit->Check(SHOW_SPLASH,tmp);

    wxConfig::Get()->Read("AutoRename Instances",&m_autoRename);
    edit->Check(AUTORENAME,m_autoRename);

    // Help menu
    wxMenu *help = new wxMenu();

    help->Append(HELP_ABOUT,_("&About"));

    // Put it all together
    menuBar->Append(file,      _("&File"));
    menuBar->Append(edit,      _("&Edit"));
    menuBar->Append(help,      _("&Help"));

    SetMenuBar(menuBar);

    updateStatusCount();
    setupToolbar();
    updateMenu();
}


/***************************************************************************/
/*!
 *  Shows the about box
 *
 ***************************************************************************/
void frame::onAbout(wxCommandEvent &event)
{
    wxString aboutText = wxString::Format(_("VistaLab Version %s\n\n\
This program is released under the terms of the\n\
VistaLab Public License V1.5.\n\n(c)\
Fabian Wenzel 2001 - 2005"),VERSION);

    wxMessageBox(aboutText,_("About this program"),wxICON_INFORMATION|wxCENTRE|wxOK);
}


/***************************************************************************/
/*!
 *  Sets the slider to the max value and starts the processor
 *
 ***************************************************************************/
void frame::processingStart(wxCommandEvent &event)
{
    m_slider->SetValue(m_slider->GetMax());
    m_slice  =  m_slider->GetMax();
    m_notebook->SetSelection(0);

    wxGetApp().getProcessor()->start();

}


/***************************************************************************/
/*!
 *  Stops the processor
 *
 ***************************************************************************/
void frame::processingStop(wxCommandEvent &event)
{
    GetToolBar()->ToggleTool( PROCESS_PAUSE, false);
    wxGetApp().getProcessor()->stop();
}


/***************************************************************************/
/*!
 *  Calls the global options dialog and ajusts the slider to the new
 *  bufferLength
 *
 ***************************************************************************/
void frame::globalOptions(wxCommandEvent &event)
{
    moduleConfigDialog *dialog =
        new moduleConfigDialog();
    dialog->attachModule(wxGetApp().getDocument());
    dialog->configure();
    delete dialog;
}


void frame::onReloadModules(wxCommandEvent &event)
{
    if(wxGetApp().getProcessor()->isStopped()){
        projectConfigXml projectConfig(NULL);

        // Save current modules to temporary XML tree
        projectConfig.prepareGlobalGroup("Modules");

        TActiveModules modules = wxGetApp().getProcessor()->getModules();
        TActiveModules::iterator currentModule=modules.begin();

        for (int i=0;currentModule!=modules.end();
            currentModule++,i++){
            // Write each input channel
            projectConfig.writeNextModule((*currentModule)->getName().c_str());
            moduleConfigFileBase &moduleConfig = projectConfig.getConfigurizer();
            moduleConfig.attachModule(*currentModule);
            moduleConfig.save();
        }

        // Remove them
        wxGetApp().getProcessor()->removeAll();

        // Remove repository
        m_repositoryTree->cleanRepository();
        wxGetApp().getRepository()->cleanModules();

#ifdef WIN32
        // Show dialog for overwriting now
        wxMessageBox(_("Ready for reload. Click OK when ready"),_("Info"),wxOK|wxCENTRE);
#endif

        wxGetApp().getRepository()->registerModules();
        m_repositoryTree->initRepository();

        // Rewind for read
        projectConfig.prepareGlobalGroup("Modules");

        bool successfulLoading=true;

        while(projectConfig.hasNextModule() && successfulLoading){
            const char *moduleName = projectConfig.readNextModule();
            if(moduleName)
                successfulLoading=addModule(moduleName,projectConfig.getConfigurizer(),NULL,true);
        }
    }
}


/***************************************************************************/
/*!
 *  Opens a file using the given fileName. This takes place by using
 *  a wxFileConfig - object and looping through the sections for
 *  input streams and processors as well as global options
 *
 ***************************************************************************/
void frame::projectOpenFile(const wxString &fileName)
{
    if(!wxFileExists(fileName)){
        wxMessageBox(_("File not found: ")+fileName,_("Error"),wxICON_ERROR|wxCENTRE|wxOK);
        return;
    }

    projectClose();

    projectConfig *projectConfig;
    if(fileName.Matches("*.vxp"))
        projectConfig = new projectConfigXml(fileName.c_str());
    else if (fileName.Matches("*.vis"))
        projectConfig = new projectConfigWxFile(fileName.c_str());
    else {
        wxMessageBox(_("Unknown file extension."),_("Error"),wxICON_ERROR|wxCENTRE|wxOK);
        return;
    }

    // Temporarily change working directory such relative path names can be found
    wxString cwd=wxGetCwd();
    wxSetWorkingDirectory(wxPathOnly(fileName));

    projectConfig->prepareGlobalGroup("Modules");

    bool successfulLoading=true;

    while(projectConfig->hasNextModule() && successfulLoading){
        const char *moduleName = projectConfig->readNextModule();
        if(moduleName)
            successfulLoading=addModule(moduleName,projectConfig->getConfigurizer(),NULL,true);
    }


    projectConfig->prepareGlobalGroup("GlobalSettings",0);

    moduleConfig &globalconfig = projectConfig->getConfigurizer();
    globalconfig.attachModule(wxGetApp().getDocument());
    globalconfig.configure();

    wxGetApp().getDocument()->postConfig();

    m_currentProject = fileName;
    SetTitle(wxGetApp().GetAppName()+wxString(" - ")+fileName);

    delete projectConfig;
    wxSetWorkingDirectory(cwd);

    m_activeModuleList->setToBeSaved(false);
}


/***************************************************************************/
/*!
 *  Saves a project file to disk. This is done by creating a
 *  a wxFileConfig - object and looping through the sections for
 *  input streams and processors as well as global options
 *
 ***************************************************************************/
bool frame::projectSaveFile(wxString &fileName)
{
    if (!fileName.Matches("*.vxp")){
        int result = wxMessageBox(_("Your fileName does not seem to \
be a VistaLab XML project. Do you want \
to change it to the XML file type \
as the old format will not be supported \
in the future?"),_("Save in XML format?"),wxYES_NO|wxCANCEL|wxCENTRE);
        if(result == wxCANCEL)
            return false;
        if(result == wxYES)
            fileName = fileName.Mid(0,fileName.Length()-4)+".vxp";
    }

    if (wxFileExists(fileName)){
        if(!wxFile::Access(fileName,wxFile::write)){
            wxMessageBox(_("Cannot write project file.\nPlease check\
 permissions!"),_("Error"),wxICON_ERROR|wxCENTRE|wxOK);
            return false;
        }
        wxRemoveFile(fileName);
    }

    // Temporarily change working directory such path names can be relative
    wxString cwd=wxGetCwd();
    wxSetWorkingDirectory(wxPathOnly(fileName));

    projectConfig *projectConfig;
    if (fileName.Matches("*.vxp"))
        projectConfig = new projectConfigXml(fileName.c_str());
    else if (fileName.Matches("*.vis"))
        projectConfig = new projectConfigWxFile(fileName.c_str());
    else {
        wxMessageBox(_("Unknown file format!"),_("Error"),wxICON_ERROR|wxCENTRE|wxOK);
        wxSetWorkingDirectory(cwd);
        return false;
    }

    projectConfig->prepareGlobalGroup("Modules");

    TActiveModules modules = wxGetApp().getProcessor()->getModules();
    TActiveModules::iterator currentModule=modules.begin();

    for (int i=0;currentModule!=modules.end();
        currentModule++,i++){
            // Write each input channel
            projectConfig->writeNextModule((*currentModule)->getName().c_str());
            moduleConfigFileBase &moduleConfig = projectConfig->getConfigurizer();
            moduleConfig.attachModule(*currentModule);
            moduleConfig.save();
        }

    projectConfig->prepareGlobalGroup("GlobalSettings");
    moduleConfigFileBase &globaloptionsconf = projectConfig->getConfigurizer();
    globaloptionsconf.attachModule(wxGetApp().getDocument());
    globaloptionsconf.save();

    delete projectConfig;

    m_currentProject = fileName;
    SetTitle(wxGetApp().GetAppName()+wxString(" - ")+fileName);

    m_activeModuleList->setToBeSaved(false);

    wxSetWorkingDirectory(cwd);
    return true;
}


/***************************************************************************/
/*!
 *  Opens a project using a fileselector. This calls the helper function
 *  afterwards
 *
 ***************************************************************************/
void frame::projectOpen(wxCommandEvent &event)
{
    wxString path;

    if(m_currentProject != wxEmptyString)
        path = wxPathOnly(m_currentProject);
    else if(!m_recentProjects.empty())
        path = wxPathOnly(m_recentProjects.front());

    wxString fileName =  wxFileSelector(_("Open project file"),
        path,wxEmptyString,"*.vxp",
        _("VistaLab XML project|*.vxp|VistaLab project|*.vis|All files|*.*"),
        wxOPEN|wxFILE_MUST_EXIST);

    if(fileName != wxEmptyString)
        projectOpenFile(fileName);
}


/***************************************************************************/
/*!
 *  Saves a project using a fileselector. This calls the helper function
 *  afterwards
 *
 ***************************************************************************/
void frame::onProjectSaveAs(wxCommandEvent &event)
{
    projectSaveAs();
}

/***************************************************************************/
/*!
 *  Saves a project using a fileselector. This calls the helper function
 *  afterwards
 *
 ***************************************************************************/
bool frame::projectSaveAs()
{
    wxFileDialog saveAsDialog(this,_("Save project file"),wxEmptyString,wxEmptyString,
                    _("VistaLab XML project|*.vxp|VistaLab project|*.vis|All files|*.*"),
                    wxSAVE|wxOVERWRITE_PROMPT);

    if(saveAsDialog.ShowModal() == wxID_OK){
        wxString fileName =  saveAsDialog.GetPath();

        if(fileName == wxEmptyString)
            return false;

        if(!fileName.Matches("*.*"))
            switch(saveAsDialog.GetFilterIndex()){

            case 1: fileName+=".vis"; break;
            case 2:
            case 0: fileName+=".vxp";

            }

        return projectSaveFile(fileName);
    }

    return false;
}

/***************************************************************************/
/*!
 *  Saves a project
 *
 ***************************************************************************/
void frame::onProjectSave(wxCommandEvent &event)
{
    if(m_currentProject != wxEmptyString)
        projectSaveFile(m_currentProject);
    else
        projectSaveAs();
}


/***************************************************************************/
/*!
 *  Updates the start/stop - menu and the slider depending on state
 *
 ***************************************************************************/
void frame::updateMenu(){
    wxMenuBar *menubar = GetMenuBar();

    menubar->Enable( VISTALABFILE_OPEN ,      !wxGetApp().getProcessor()->isRunning());
    menubar->Enable( VISTALABFILE_CLOSE,      !wxGetApp().getProcessor()->isRunning());
    menubar->Enable( VISTALABFILE_SAVE,       !wxGetApp().getProcessor()->isRunning());
    menubar->Enable( PROCESS_START,   wxGetApp().getProcessor()->isStopped()
        && wxGetApp().getProcessor()->getModules().size() > 0);
    menubar->Enable( PROCESS_STOP,     wxGetApp().getProcessor()->isRunning());
    menubar->Enable( EDIT_OPTIONS,    !wxGetApp().getProcessor()->isRunning());
    menubar->Enable( UNITTEST_MODULES,!wxGetApp().getProcessor()->isRunning());

    wxToolBar *toolBar = GetToolBar();
    toolBar->EnableTool( VISTALABFILE_OPEN ,  !wxGetApp().getProcessor()->isRunning());
    toolBar->EnableTool( VISTALABFILE_CLOSE,  !wxGetApp().getProcessor()->isRunning());
    toolBar->EnableTool( VISTALABFILE_SAVE,   !wxGetApp().getProcessor()->isRunning());
    toolBar->EnableTool( PROCESS_START,wxGetApp().getProcessor()->isStopped()
        && wxGetApp().getProcessor()->getModules().size() > 0);
    toolBar->EnableTool( PROCESS_PAUSE, wxGetApp().getProcessor()->isRunning());
    toolBar->EnableTool( PROCESS_STOP, wxGetApp().getProcessor()->isRunning());
    toolBar->EnableTool( EDIT_OPTIONS,!wxGetApp().getProcessor()->isRunning());
    toolBar->EnableTool( UNITTEST_MODULES,!wxGetApp().getProcessor()->isRunning());

    m_slider->Enable( !wxGetApp().getProcessor()->isRunning());
}


/***************************************************************************/
/*!
 *  Callback: If the slider has been moved
 *
 ***************************************************************************/
void frame::onSlider(wxCommandEvent &e)
{
    const int newSlice=m_slider->GetValue();

    while(m_slice < newSlice ){
        vistaLab::processCore::shiftFrameCounters(1);
        wxGetApp().getProcessor()->
            showNeighborSlice(false);
        m_slice++;
    }

    while(m_slice > newSlice ){
        vistaLab::processCore::shiftFrameCounters(-1);
        wxGetApp().getProcessor()->
            showNeighborSlice(true);
        m_slice--;
    }

    updateFrameTickCounter();
}



/***************************************************************************/
/*!
 *  Updates the count of input channels / processors in the status bar
 *
 ***************************************************************************/
void frame::updateStatusCount()
{
    GetStatusBar()->SetStatusText(wxString::Format(_("%d Modules"),
        wxGetApp().getProcessor()->getModules().size()),3);
}


/***************************************************************************/
/*!
 *  Callback: If the user wants to close the whole project
 *
 ***************************************************************************/
void frame::onProjectClose(wxCommandEvent &event)
{
    projectClose();
}


/***************************************************************************/
/*!
 *  Gets called either by the menu or when quitting the application
 *
 ***************************************************************************/
bool frame::projectClose()
{
    if(m_activeModuleList->needsToBeSaved())
    {
        if(m_currentProject == wxEmptyString)
        {
            int result = wxMessageBox(_("Save as new project?"),_("Close project"),wxYES_NO|wxCANCEL|wxCENTRE);
            if (result == wxCANCEL || (result == wxYES && !projectSaveAs()))
                return false;
        }
        else
        {
            int result = wxMessageBox(wxString::Format(_("Save changes to %s?"),
                m_currentProject.c_str()),_("Close project"),wxYES_NO|wxCANCEL|wxCENTRE);
            if (result == wxCANCEL || (result == wxYES && !projectSaveFile(m_currentProject)))
                return false;
        }
        m_activeModuleList->setToBeSaved(false);
    }
    wxGetApp().getProcessor()->removeAll();
    if ( m_currentProject != wxEmptyString )
        insertRecentProject( m_currentProject );
    m_currentProject=wxEmptyString;
    SetTitle(wxGetApp().GetAppName());
    m_activeModuleList->updateModules();
    updateStatusCount();
    return true;
}


/***************************************************************************/
/*!
 *  Helper function: Adds a chosen module using the passed configuration
 *  method. This gets called both by the GUI when clicking on a menu in
 *  frame::onMenu() or when loading from a file and detecting a module
 *  in frame::projectOpen()
 *
 ***************************************************************************/
bool frame::addModule(const wxString &itemName, moduleConfig &config,
                      module *predecessor, bool reconfigure)
{

    // Try to find the module in the module - repository
    TModuleRepository modules = wxGetApp().getRepository()->getModules();
    TModuleRepository::iterator currentModule;
    for (currentModule  = modules.begin();
         currentModule != modules.end();
        ++currentModule){

            // The localized item name has to be checked as well as the non-localized
            // one as the identifier of the module may come from file or from menu
            if(_((*currentModule)->getName().c_str()) == itemName ||
                 (*currentModule)->getName().c_str()  == itemName ){
                    // The module has been found. If configuring is successful
                    // Add it to the view and the processor

                    config.attachModule(*currentModule);
                    bool result=config.configure();

                    if(!result && reconfigure){
                        wxMessageBox(wxString::Format(_("Could not add module \"%s\":\n\n\
Configuration failed. Please configure now."),
                        _((*currentModule)->getName().c_str())),_("Warning"),wxOK|wxICON_EXCLAMATION);

                        moduleConfigDialog config;
                        config.attachModule(*currentModule);
                        result=config.configure();
                    }

                    if(result || dynamic_cast<moduleConfigFile *>(&config)){
                        wxGetApp().getProcessor()->addModule(*currentModule,predecessor);
                        updateStatusCount();
                        updateMenu();
                        m_activeModuleList->updateModules();
                        if(!dynamic_cast<moduleConfigFile *>(&config)){
                            // We are configuring via dialog, so project needs to be saved
                            m_activeModuleList->setToBeSaved(true);
                        }
                        return true;
                    }

                    if( !result ){
                        wxString error(wxString::Format(_("Could not add module \"%s\":\n\n\
Configuration canceled."),_((*currentModule)->getName().c_str())));

                        if(dynamic_cast<moduleConfigFile *>(&config)){
                            error+=_(" Stop further loading.");
                        }
                        wxMessageBox(error,_("Error"),wxOK|wxICON_ERROR);
                    }

                    return false;
                }
        }

    wxMessageBox(wxString::Format(_("Could not add module \"%s\":\n\nNot present in repository. Aborting loading."),
        itemName.c_str()),_("Error"),wxOK|wxICON_ERROR);

    return false;
}



/***************************************************************************/
/*!
 *  Loads up to four recent files from config file
 *
 ***************************************************************************/
void frame::loadRecentProjects()
{
    int i;
    wxConfigBase *config = wxConfig::Get();
    config->SetPath("/Recent projects");
    m_recentProjects.clear();
    for (i=0;i<NUM_RECENTPROJECTS;i++){
        wxString tmpRecentProject;
        if(config->Read(wxString::Format("project %d",i),&tmpRecentProject))
            m_recentProjects.push_back(tmpRecentProject);
    }

    config->SetPath("/");

    Connect(m_recentProjectMenuID,m_recentProjectMenuID+NUM_RECENTPROJECTS,
        wxEVT_COMMAND_MENU_SELECTED,
        (wxObjectEventFunction) (wxEventFunction)
        (wxCommandEventFunction) &frame::onRecentProject);

}


/***************************************************************************/
/*!
 *  Saves up to four recent files from config file
 *
 ***************************************************************************/
void frame::saveRecentProjects()
{
    int i;
    wxConfigBase *config = wxConfig::Get();
    config->SetPath("/Recent projects");
    list< wxString >::const_iterator currentRecentProject;
    for (i=0,currentRecentProject=m_recentProjects.begin();
        currentRecentProject!= m_recentProjects.end();
        ++currentRecentProject,i++){
            config->Write(wxString::Format("project %d",i),*currentRecentProject);
        }
        config->Flush();
}


/***************************************************************************/
/*!
 *  Updates the file menu with the correct recent projects
 *
 ***************************************************************************/
void frame::updateRecentProjects()
{
    int i;
    list< wxString >::const_iterator currentRecentProject;
    // Loop through all recent projects
    for (i=0,currentRecentProject=m_recentProjects.begin();
        currentRecentProject != m_recentProjects.end();
        ++currentRecentProject,i++){
            // only do something if there is an entry
            if (GetMenuBar()->FindItem(m_recentProjectMenuID+i) != NULL)
                GetMenuBar()->FindItem(m_recentProjectMenuID+i)->
                SetText(*currentRecentProject+wxString::Format("\tCtrl-%d",i+1));
            else {
                wxMenu *fileMenu = GetMenuBar()->GetMenu(0);
                // It must be appended before quit()
                // GetMenu(0) is the file - menu (always)
                fileMenu->Insert(fileMenu->GetMenuItemCount()-2,
                    new wxMenuItem(fileMenu, m_recentProjectMenuID+i,
                    *currentRecentProject+wxString::Format("\tCtrl-%d",i+1)));
            }
        }
}


/***************************************************************************/
/*!
 *  Callback when a recent file should be opened
 *
 ***************************************************************************/
void frame::onRecentProject(wxCommandEvent &event)
{
    int i=m_recentProjectMenuID;
    list< wxString >::iterator currentProject;
    for(currentProject = m_recentProjects.begin();
        i != event.GetId();
        i++,++currentProject);

    projectOpenFile(*currentProject);
}


/***************************************************************************/
/*!
 *  Inserts a new projectName into the list or recent projects
 *
 ***************************************************************************/
void frame::insertRecentProject(const wxString &projectName)
{
    list< wxString >::iterator presentProject;
    presentProject = find(m_recentProjects.begin(),
        m_recentProjects.end(),
        projectName);

    if( presentProject != m_recentProjects.end() )
        m_recentProjects.erase(presentProject);

    m_recentProjects.push_front(projectName);

    if(m_recentProjects.size() > NUM_RECENTPROJECTS )
        m_recentProjects.resize(NUM_RECENTPROJECTS);

    updateRecentProjects();
}


/***************************************************************************/
/*!
 *  Sets up the toolBar for the frame
 *
 ***************************************************************************/
void frame::setupToolbar()
{
    wxToolBar *toolBar = CreateToolBar(wxTB_HORIZONTAL | wxTB_FLAT    |
        wxTB_TEXT       | wxTB_DOCKABLE);

    toolBar->SetToolBitmapSize(wxSize(22,22));

    toolBar->AddTool(VISTALABFILE_CLOSE,_("New"),
        wxBitmap(newproject_xpm),
        _("New project"));
    toolBar->AddTool(VISTALABFILE_OPEN,_("Open"),
        wxBitmap(openproject_xpm),
        _("Opens project from file"));
    toolBar->AddTool(VISTALABFILE_SAVE,_("Save"),wxBitmap(saveproject_xpm),
        _("Saves project to file"));
    toolBar->AddSeparator();
    toolBar->AddTool(EDIT_OPTIONS, _("Config") ,wxBitmap(configure_xpm),
        _("Global Configuration Options"));
    toolBar->AddTool(UNITTEST_MODULES,_("Test"),wxBitmap(testmodules_xpm),
        _("Run all available Unit Tests"));
    toolBar->AddSeparator();
    toolBar->AddTool(PROCESS_START,_("Start"),wxBitmap(run_xpm),
        _("Start processing"));
    toolBar->AddCheckTool(PROCESS_PAUSE,_("Pause"),wxBitmap(pause_xpm),wxNullBitmap,
        _("Pause processing"));
    toolBar->AddTool(PROCESS_STOP ,_("Stop" ),wxBitmap(stop_xpm),
        _("Stop processing"));
    toolBar->Realize();
}



/***************************************************************************/
/*!
 *  Callback for Unit Tests on all modules in the repository
 *
 ***************************************************************************/
void frame::onUnitTest(wxCommandEvent &event)
{
    // Generate log window
    wxDialog *dialog = new wxDialog(NULL,-1,_("Unit Test"),
        wxDefaultPosition,wxSize(320,480),
        wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);

    wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);
    dialog->SetSizer(topSizer);

    wxTextCtrl *textCtrl = new wxTextCtrl(dialog,-1,wxEmptyString,
        wxDefaultPosition,wxDefaultSize,
        wxTE_MULTILINE);

    topSizer->Add(textCtrl,wxSizerFlags(1).Expand().Border(wxALL,5));

    topSizer->Add(new wxButton(dialog,wxID_OK,_("OK")),wxSizerFlags(0).Center().Border(wxALL,10));

    // Iterate through all modules and run unit tests
    TModuleRepository modules = wxGetApp().getRepository()->getModules();
    TModuleRepository::iterator currentModule = modules.begin();

    while (currentModule  != modules.end())
    {
        TConfigItems::const_iterator currentItem = (*currentModule)->getConfigItems()->begin();

        bool founditem = false;
        while (currentItem != (*currentModule)->getConfigItems()->end())
        {
            if(currentItem->m_type == configItem::UNITTESTPIN)
            {
                textCtrl->SetForegroundColour(*wxBLACK);
                if(!founditem){
                    founditem = true;
                    textCtrl->AppendText(_((*currentModule)->getName().c_str())+wxString(":\n\n"));
                }
                textCtrl->AppendText(wxString(currentItem->m_localizedDescription.c_str())+wxString::Format(" - "));

                if(((*currentModule)->*((configItem::TUnitTest) currentItem->m_target.func))())
                {
                    textCtrl->SetForegroundColour(*wxGREEN);
                    textCtrl->AppendText(_("OK\n"));
                } else {
                    textCtrl->SetForegroundColour(*wxRED);
                    textCtrl->AppendText(_("Error\n"));
                }
            }
            ++currentItem;
        }
        ++currentModule;
    }

    textCtrl->SetForegroundColour(*wxBLACK);
    textCtrl->AppendText(_("\n----- done."));
    dialog->ShowModal();

    delete dialog;
}


/***************************************************************************/
/*!
 *  Callback for Splashscreen option
 *
 ***************************************************************************/
void frame::onShowSplash(wxCommandEvent &event)
{
    wxConfig::Get()->Write("ShowSplash",event.IsChecked());
}


/***************************************************************************/
/*!
 *  Callback for Autorename option
 *
 ***************************************************************************/
void frame::onAutoRename     (wxCommandEvent &event)
{
    m_autoRename=event.IsChecked();
}


/***************************************************************************/
/*!
 *  Callback for changing the internal geometry (using sash windows)
 *
 ***************************************************************************/
void frame::onSashDrag(wxSashEvent& event)
{
    if (event.GetDragStatus() == wxSASH_STATUS_OUT_OF_RANGE)
        return;

    m_sash->SetDefaultSize(wxSize(event.GetDragRect().width, 1000));

    wxLayoutAlgorithm layout;
    layout.LayoutFrame(this,m_notebook);

    m_notebook->Layout();
}


/***************************************************************************/
/*!
 *  Callback for changing the size (has to be overwritten as the standard
 *  behavior - slider or constraints - is not used due to sash windows)
 *
 ***************************************************************************/
void frame::onSize(wxSizeEvent & WXUNUSED(event))
{

    wxLayoutAlgorithm layout;
    layout.LayoutFrame(this,m_notebook);

    m_notebook->Layout();
}


/***************************************************************************/
/*!
 *  Gets called by postConfig() of vistaLabDocument, i.e. whenever the buffer-
 *  size might have changed.
 *
 ***************************************************************************/
void frame::updateSlider()
{
    m_slider->SetRange(0,wxGetApp().getDocument()->getBufferLength()-1);
    if (m_slider->GetValue() > wxGetApp().getDocument()->getBufferLength()-1 )
        m_slider->SetValue(wxGetApp().getDocument()->getBufferLength()-1);
    m_topSizer->Layout();
}


/***************************************************************************/
/*!
 *  Updates the frameCounter status cells
 *
 ***************************************************************************/
void frame::updateFrameTickCounter()
{
    int framedifference = processCore::getTriggeredFrames()-
                          module::currentFrameNumber();

    GetStatusBar()->SetStatusText(wxString::Format(_("Missed %d frames"),
        framedifference),1);
    GetStatusBar()->SetStatusText(wxString::Format(_("Frame %d"),
        module::currentFrameNumber()),2);
}


/***************************************************************************/
/*!
 *  When being closed, first remove all active modules (so that they
 *  can close themselves before this window gets destroyed... some
 *  have GUI elements which would be destroyed automatically here
 *  otherwise so that the destructors of the modules can't operate
 *  as they would try to delete once again)
 *
 ***************************************************************************/
void frame::onClose(wxCloseEvent   &event)
{
    if(projectClose())
        Destroy();
}

bool frame::getAutoRename()
{
    return m_autoRename;
}

void frame::processingPause(wxCommandEvent &event)
{
    wxToolBar *toolBar = GetToolBar();
    bool toggle = toolBar->GetToolState(PROCESS_PAUSE);
    wxGetApp().getProcessor()->setPause(toggle);
}

void frame::onDisplayError(wxCommandEvent &event)
{
    wxString error = getError().c_str();

    if(error != wxEmptyString)
        wxMessageBox(error,
            _("Error"),wxOK|wxICON_ERROR);

    setError("");
}

void frame::onUpdateTicks(wxCommandEvent &event)
{
    updateFrameTickCounter();
}
