/***************************************************************************
                         vlframe.h  -  description
                            -------------------
    begin                : Fri Nov 16 2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLFRAME_H_
#define _VLFRAME_H_


#include <wx/frame.h>
#include <wx/notebook.h>

#include "vlview.h"
#include "vlmodule.h"


#define NUM_RECENTPROJECTS 4

class wxSlider;
class wxSashLayoutWindow;
class wxSashEvent;

namespace vistaLab {
extern const wxEventType wxEVT_VISTALAB;

// Event IDs
enum{
    PROCESS_START=wxID_HIGHEST+1,
    PROCESS_STOP,
    PROCESS_PAUSE,
    REMOVE_MODULE,
    CONFIGURE_MODULE,
    UNITTEST_MODULES,
    VISTALABFILE_OPEN,
    VISTALABFILE_SAVE,
    VISTALABFILE_SAVEAS,
    VISTALABFILE_CLOSE,
    VISTALABFILE_RECENT,
    EDIT_OPTIONS,
    EDIT_REFRESH,
    SHOW_SPLASH,
    AUTORENAME,
    HELP_ABOUT,
    ID_FRAME_SLIDER,
    ID_REPOSITORYTREE,
    ID_ACTIVEMODULELIST,
    ID_SASH,
    ID_NOTEBOOK,
    DISPLAY_ERROR,
    UPDATE_TICKS
};

class repositoryTree;
class activeProjectTree;
class moduleConfig;

/***************************************************************************/
//! The frame window
/*!
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class frame : public wxFrame {
    friend class repositoryTree;
    friend class activeProjectTree;
    friend class moduleDropTarget;
    friend class processor;
public:
    frame(wxWindow* parent, wxWindowID id, const wxString& title);
    ~frame();

    // Function for accessing the view child (and its panel which is the
    // scrolled child of the view)
    view *getChannelView       ()      { return m_view; }
    void updateSlider               ();
    void updateFrameTickCounter     ();
    bool getAutoRename              ();

    const wxString &getCurrentProject()
    {
        return m_currentProject;
    }

    activeProjectTree *getActiveProjectTree()
    {
        return m_activeModuleList;
    }

    // Helper functions for opening and saving projects
    void projectOpenFile(const wxString &fileName);
    bool projectSaveFile(wxString &fileName);
    bool projectSaveAs  ();
    bool projectClose   ();

private:

    // Callback functions for menu items
    void onRecentProject  (wxCommandEvent &event);
    void onQuit           (wxCommandEvent &event);
    void onClose          (wxCloseEvent   &event);
    void onAbout          (wxCommandEvent &event);
    void processingStart  (wxCommandEvent &event);
    void processingPause  (wxCommandEvent &event);
    void processingStop   (wxCommandEvent &event);
    void globalOptions    (wxCommandEvent &event);
    void onReloadModules  (wxCommandEvent &event);
    void projectOpen      (wxCommandEvent &event);
    void onProjectSaveAs  (wxCommandEvent &event);
    void onProjectSave    (wxCommandEvent &event);
    void onProjectClose   (wxCommandEvent &event);
    void onSlider         (wxCommandEvent &event);
    void onUnitTest       (wxCommandEvent &event);
    void onShowSplash     (wxCommandEvent &event);
    void onAutoRename     (wxCommandEvent &event);
    void onSashDrag       (wxSashEvent& event   );
    void onSize           (wxSizeEvent & WXUNUSED(event));
    void onDisplayError   (wxCommandEvent &event);
    void onUpdateTicks    (wxCommandEvent &event);


    // Misc functions
    bool addModule(const wxString &itemname,moduleConfig &config,
        module *predecessor=NULL, bool reconfigure=false);
    void setupMenu();
    void setupToolbar();
    void loadRecentProjects  ();
    void saveRecentProjects  ();
    void updateRecentProjects();
    void insertRecentProject (const wxString &projectName);

    // Helper functions for updating the GUI elements
    void updateMenu();
    void updateStatusCount  ();

    // The current slice number
    int m_slice;

    // Saves autorename config option
    bool m_autoRename;

    // The child elements (and an ID counter for the menus)
    int         m_lowestMenuID;
    const int   m_recentProjectMenuID;
    wxMenuBar  *m_vistaLabMenuBar;
    wxBoxSizer *m_topSizer;

    wxSashLayoutWindow     *m_sash;
    wxNotebook             *m_notebook;
    repositoryTree         *m_repositoryTree;
    activeProjectTree      *m_activeModuleList;

    view                   *m_view;
    wxPanel                *m_sliderViewPanel;
    wxSlider               *m_slider;

    // The last loaded project files for the file menu
    std::list< wxString > m_recentProjects;
    wxString              m_currentProject;

    DECLARE_EVENT_TABLE()
};

} // namespace vistaLab

#endif // #ifndef _VLFRAME_H_
