/***************************************************************************
                     vlframecontrols.h  -  description
                            -------------------
    begin                : Aug 15 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLFRAMECONTROLS_H_
#define _VLFRAMECONTROLS_H_

#include <list>
#include <vector>
#include <wx/listctrl.h>
#include <wx/treectrl.h>
#include "vlprocessor.h"

namespace vistaParameterItems{
    class parameterItemBase;
    typedef std::vector< parameterItemBase *> TParameterChanges;
}

namespace vistaLab {

/***************************************************************************/
//! TreeCtrl for a tree-view of available modules
/*!
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class repositoryTree : public wxTreeCtrl{
public:
    repositoryTree(wxWindow *parent, const wxWindowID id);
    void initRepository();
    void cleanRepository();

private:
    enum{
        moduleCategory,
        openModuleCategory,
        moduleIcon
    };

    void onDrag     (wxTreeEvent  &event);
    void onAddModule(wxTreeEvent  &event);
    void onMouse    (wxMouseEvent &event);
    wxTreeItemId currentId;
    DECLARE_EVENT_TABLE();

    std::list< wxTreeItemId > m_categories;
};


class moduleDropTarget;


/***************************************************************************/
//! ListCtrl for active modules
/*!
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class activeProjectTree : public wxTreeCtrl{
    friend class moduleDropTarget;
public:
    activeProjectTree(wxWindow *parent, const wxWindowID id);

    void updateModules();
    void updateModule(const wxTreeItemId &id);

    bool needsToBeSaved()
    {
        return m_changed;
    }

    void setToBeSaved(bool save)
    {
        m_changed = save;
    }
    void addPendingParameterChange(vistaParameterItems::parameterItemBase *change);

    const wxString applyPendingChanges();
    bool hasParameterChanges();

    // Internal helper that applies pending parameter changes
    // if not running
    bool tryApplyChanges();


private:

    // Callbacks for the GUI

    void onConfigureDoubleClick(wxTreeEvent &event);

    void onMouseMotion         (wxMouseEvent &event);

    void onConfigure           (wxCommandEvent &event);
    void onBeginLabelEdit      (wxTreeEvent    &event);
    void onEndLabelEdit        (wxTreeEvent    &event);
    void onRemove              (wxCommandEvent &event);
    void onUpdateModules       (wxCommandEvent &event);
    void onRightClick          (wxTreeEvent    &event);
    void onBeginDrag           (wxTreeEvent    &event);
    void onEndDrag             (wxTreeEvent     &event);
    void setActive             (wxCommandEvent &event);
    void onChar                (wxKeyEvent     &event);

    DECLARE_EVENT_TABLE();

    void addModule(int x, int y, const wxString &data);

    // Stores the ID when beginning changing module order via DND
    wxTreeItemId m_draggedItem;

    // Stores the module ID when rightclicking (for later menu callbacks)
    wxTreeItemId m_rightClickItem;

    // Indicated whether the modules have changed (so that a project should be saved when exiting)
    bool m_changed;

    // Item over which the mouse is positioned (for tool tip changes)
    wxTreeItemId m_mouseOverId;

    vistaParameterItems::TParameterChanges m_pendingParameterChanges;
    wxCriticalSection m_pendingParameterLock;

    bool m_parameterChangesAreMarked;
    void markParameterChanges(bool doMark);

    // For receiving Objects from the treectrl
    moduleDropTarget *dropTarget;
    wxColour m_unmarkedItemColor;
    wxColour m_markedItemColor;
};

} // namespace vistaLab

#endif // #ifndef _VLFRAMECONTROLS_H_
