/***************************************************************************
                  vlmoduleconfigdialog.h  -  description
                            -------------------
    begin                : Mon Feb 4 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLMODULECONFIGDIALOG_H_
#define _VLMODULECONFIGDIALOG_H_

#include "vlmoduleconfig.h"
#include <wx/event.h>
#include <wx/notebook.h>
#include <vector>

class wxDialog;
class wxSizer;
class wxButton;


namespace vistaLabDialogItems{
    class configDialogItemBase;
}

namespace vistaLab {

/***************************************************************************/
//! The dialog backend for the configuration implementation using wxWidgets
/*!
 *  \author  Fabian Wenzel
 *
 ***************************************************************************/
class moduleConfigDialog : public wxEvtHandler, public moduleConfig {
public:
    typedef std::vector< vistaLabDialogItems::configDialogItemBase *>
        TDialogItems;

    moduleConfigDialog();
    virtual ~moduleConfigDialog();
    virtual void attachModule(configurableObject *module);

    // The "actions" - interface
    bool configure ();

private:
    //! Callback for the OK-Button
    void validate      (wxCommandEvent &event);

    //! Callback for the Show-Info - Button
    void showInfo      (wxCommandEvent &event);

    //! Filters out "Return"-Keys for simulating OK-Button
    void onCharHook   (wxKeyEvent    &event);

    //! For focusing the OK button when first shown
    void onIdle       (wxIdleEvent   &event);

    //  Helper functions which perform atomic tasks which each dialog item
    void save   ();
    void restore();
    void addConfigItems();

    //! Counter which gets incremented for each widget which needs an ID
    int  m_wxId;
    bool m_hasPins;

    // GUI members
    wxDialog *m_dialog;
    wxSizer  *m_dialogSizer;
    std::vector< wxPanel * > m_dialogPanels;
    wxNotebook *m_notebook;
    wxWindow *m_wantsFocus;

    //! Stores all dialog items which get queried from configManager
    TDialogItems     m_dialogItems;
};

} // namespace vistaLab

#endif // #ifndef _VLMODULECONFIGDIALOG_H_
