/***************************************************************************
                          vlpin.h  -  description
                            -------------------
    begin                : Tue Jul 23 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLPIN_H_
#define _VLPIN_H_

#include "vlconfigitem.h"
#include <string>

namespace vistaLab {

class module;

/***************************************************************************/
//! Encapsulates an active renderpin - each slice slot corresponds to one
/*!
 * \ingroup Datatypes
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class renderPin {
public:
    renderPin(module *moduleInstance,
        const configItem::TRenderFunc renderFunction,
        const std::string &description) :
    m_moduleInstance(moduleInstance),
    m_renderFunction(renderFunction),
    m_description(description) {}

    //! The pointer to the corresponding active rendering module
    module                      *m_moduleInstance;

    //! The pointer to the corresponding renderFunction of the active module
    const configItem::TRenderFunc  m_renderFunction;

    //! Information about the renderFunction which gets displayed in the dialog
    const std::string                  m_description;
};


/***************************************************************************/
//! Encapsulates an active logpin
/*!
 * \ingroup Datatypes
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class logPin {
public:
    logPin(module *moduleInstance,
        const configItem::TLogFunc logFunction,
        const std::string &description) :
    m_moduleInstance(moduleInstance),
    m_logFunction(logFunction),
    m_description(description) {}

    //! The pointer to the corresponding active rendering module
    module                      *m_moduleInstance;

    //! The pointer to the corresponding logFunction of the active module
    const configItem::TLogFunc   m_logFunction;

    //! Information about the renderFunction which gets displayed in the dialog
    const std::string            m_description;
};


/***************************************************************************/
//! Encapsulates an active outputpin - each slice slot corresponds to one
/*!
 * \ingroup Datatypes
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class outputPin {
public:
    outputPin(module *moduleInstance,
        const std::string &type,
        const std::string &description, int slot) :
    m_moduleInstance(moduleInstance),
    m_type(type),
    m_description(description),m_slot(slot) {}

    //! The pointer to the corresponding active module
    const module *m_moduleInstance;

    //! String coded datatype of the slot's content (i.e. "vigra::BRGBImage")
    const std::string   m_type;

    //! Information about the semantic of the datatype (i.e. "Filtered image")
    const std::string   m_description;

    //! The slot in the slice (has to be kept extra for removing modules)
    int                 m_slot;
};

} // namespace vistaLab

#endif // #ifndef _VLPIN_H_
