/***************************************************************************
                       vlprojectconfigwxfile.h  -  description
                            -------------------
    begin                : Tue Aug 3 2004
    copyright            : (C) 2004 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLPROJECTCONFIGWXFILE_H_
#define _VLPROJECTCONFIGWXFILE_H_

#include "vlprojectconfig.h"

namespace vistaLab {

/***************************************************************************/
//! The file backend for the configuration implementation using files
/*!
 *  \author  Fabian Wenzel
 *
 ***************************************************************************/
class projectConfigWxFile : public projectConfig {
public:
    projectConfigWxFile(const char *fileName):
    m_configIndex(0),
    m_firstModule(true)
    {
        m_oldLocale = setlocale(LC_NUMERIC,"C");

        // create config object and make it global such that
        // wxModuleConfigFile finds it
        m_config = new wxFileConfig(wxEmptyString,
            wxEmptyString,fileName,wxEmptyString,wxCONFIG_USE_LOCAL_FILE);
        m_moduleConfig = new moduleConfigFile(m_config);
    }

    ~projectConfigWxFile()
    {
        setlocale(LC_NUMERIC,m_oldLocale);
        delete m_moduleConfig;
        delete m_config;
    }

    void prepareGlobalGroup(const char *groupName, bool enterGroup=1)
    {
        m_config->SetPath("/"+wxString(groupName));
    }

    bool hasNextModule()
    {
        if(m_firstModule){
            m_firstModule = false;
            return m_config->GetFirstGroup(m_configSection,m_configIndex);
        } else{
            m_config->SetPath("..");
            return m_config->GetNextGroup(m_configSection,m_configIndex);
        }
    }

    const char *readNextModule()
    {
        wxString itemName;
        m_config->SetPath(m_configSection);
        if( m_config->Read("Itemname",&itemName) ){
            return itemName.c_str();
        }
        else
            return NULL;
    }

    void writeNextModule(const char *moduleName){
        if(!m_firstModule){
            m_configIndex++;
            m_config->SetPath("..");
        }
        m_config->SetPath(wxString::Format("Module %03ld",m_configIndex));
        m_config->Write("Itemname",wxString(moduleName));
        m_firstModule=false;
    }

    moduleConfigFileBase &getConfigurizer()
    {
        return *m_moduleConfig;
    }

private:
    const char       *m_oldLocale;
    wxFileConfig     *m_config;
    long              m_configIndex;
    wxString          m_configSection;
    moduleConfigFile *m_moduleConfig;
    bool m_firstModule;
};

} // namespace vistaLab

#endif // #ifndef _VLPROJECTCONFIGWXFILE_H_
