/***************************************************************************
                      vlprojectconfigxml.h  -  description
                            -------------------
    begin                : Wed Aug 4 2004
    copyright            : (C) 2004 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VISTALABPROJECTCONFIGXML_H_
#define _VISTALABPROJECTCONFIGXML_H_

#include "vlprojectconfig.h"
#include "vlmoduleconfigxml.h"
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
#include <wx/utils.h>

#define MY_ENCODING "ISO-8859-1"

namespace vistaLab {

/***************************************************************************/
//! The file backend for the configuration implementation using files
/*!
 *  \author  Fabian Wenzel
 *
 ***************************************************************************/
class projectConfigXml : public projectConfig {
public:
    enum EXMLmode{
        XML_MODE_READ,
        XML_MODE_WRITE
    };

    projectConfigXml(const char *fileName):
    m_fileName(fileName),
    m_mode(XML_MODE_READ),
    m_doc(0),
    m_currentGroup(0),
    m_currentModule(0),
    m_moduleConfigXML(0)
    {
        // Try reading the file, otherwise
        // Create empty document
        if(wxFileExists(m_fileName))
            m_doc = xmlParseFile(m_fileName);

        if(!m_doc){
            m_doc = xmlNewDoc(BAD_CAST "1.0");
            xmlNodePtr root_node = xmlNewNode(NULL, BAD_CAST "VistaLAB-Project");
            xmlDocSetRootElement(m_doc, root_node);
        }
    }

    ~projectConfigXml()
    {
        if(m_mode == XML_MODE_WRITE && m_fileName)
            xmlSaveFormatFileEnc(m_fileName,m_doc,MY_ENCODING,1);

        xmlFreeDoc(m_doc);
        delete m_moduleConfigXML;
    }

    void prepareGlobalGroup(const char *groupName, bool enterSettings=true)
    {
        xmlNodePtr root = xmlDocGetRootElement(m_doc);
        xmlNodePtr cur;
        for(cur=root->children;cur;cur=cur->next)
        {
            if(xmlStrEqual(cur->name,BAD_CAST groupName)){
                m_currentGroup  = cur;
                m_currentModule = enterSettings?cur->children:cur;
                m_firstModule   = true;
                return;
            }
        }

        // No global group is here, so create one
        m_currentGroup  = xmlNewChild(root, NULL, BAD_CAST groupName, NULL);
        m_currentModule = m_currentGroup;
    }

    bool hasNextModule()
    {
        if(!m_firstModule)
            m_currentModule = m_currentModule->next;

        m_firstModule=false;

        while(m_currentModule && !xmlStrEqual(m_currentModule->name,BAD_CAST "Settings")){
            m_currentModule = m_currentModule->next;
        }

        return (m_currentModule != NULL);
    }

    const char *readNextModule()
    {
        m_mode = XML_MODE_READ;
        return (const char *) xmlGetProp(m_currentModule, BAD_CAST "Itemname");
    }

    void writeNextModule(const char *moduleName)
    {
        m_mode = XML_MODE_WRITE;
        m_currentModule = xmlNewChild(m_currentGroup,NULL,BAD_CAST "Settings",NULL);
        xmlNewProp(m_currentModule, BAD_CAST "Itemname", BAD_CAST moduleName);
    }

    moduleConfigFileBase &getConfigurizer()
    {
        delete m_moduleConfigXML;
        m_moduleConfigXML = new moduleConfigXML(m_currentModule);
        return *m_moduleConfigXML;
    }

private:
    const char      *m_fileName;
    bool             m_firstModule;
    EXMLmode         m_mode;
    xmlDocPtr        m_doc;
    xmlNodePtr       m_currentGroup;
    xmlNodePtr       m_currentModule;
    moduleConfigXML *m_moduleConfigXML;
};

} // namespace vistaLab

#endif // #ifndef _VISTALABPROJECTCONFIGXML_H_
