/***************************************************************************
                       vlrepository.h  -  description
                            -------------------
    begin                : Fri Nov 23 2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLREPOSITORY_H_
#define _VLREPOSITORY_H_

#include<list>
#include<vector>

class wxDelayLine;
class wxDynamicLibrary;
class wxString;

namespace vistaLab {

class module;


typedef std::list< module   *> TModuleRepository;

/***************************************************************************/
//! The repository which stores all available modules
/*!
 *  \author  Fabian Wenzel
 *
 ***************************************************************************/
class repository
{
    friend class vistaLabGlobalOptions;
public:
    repository();
    ~repository();

    // Public members which allows access to the input streams
    const TModuleRepository &getModules() const
    {
        return m_availableModules;
    }

    void registerModules();
    void cleanModules();

private:
    // "Global" functions which adds all builtins or dynamically loads plugins
    void registerBuiltins();
    void registerPlugins();
    void initConfigs();

    // Helper functions for the plugins - they get a fileName
    void addPlugin       ( const wxString &name );

    // Basic functions which add modules
    void registerModule  ( module  *newmodule);

    // The repository members
    TModuleRepository               m_availableModules;

    // Internally used: Open libraries which have to be closed on destruction
    std::list< wxDynamicLibrary * > m_plugins;
};

} // namespace vistaLab

#endif // #ifndef _VLREPOSITORY_H_
