/***************************************************************************
                          vlview.cpp  -  description
                            -------------------
    begin                : Tue Jan 8 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "vlview.h"
#include "vlchannelwindow.h"

#include <wx/sizer.h>

using namespace vistaLab;

////////////////////////////// Public functions ////////////////////////////



/***************************************************************************/
/*!
 *  Constructor sets up sizer and panel for later adding childs
 *
 ****************************************************************************/
view::view(wxWindow* parent) :
wxScrolledWindow(parent,-1,wxDefaultPosition,wxDefaultSize,wxSUNKEN_BORDER)
{

    m_panel = new wxPanel(this,-1);
    m_channelSizer = new wxBoxSizer(wxHORIZONTAL);

    m_panel->SetSizer(m_channelSizer);

    m_channelSizer->SetSizeHints(m_panel);
    m_channelSizer->Fit(m_panel);

    wxColour backgroundColor = wxTheColourDatabase->Find("LIGHT GRAY");

    m_panel->SetBackgroundColour(backgroundColor);
    SetBackgroundColour(backgroundColor);
}


/***************************************************************************/
/*!
 *  Adds a channel and resizes itself
 *
 ****************************************************************************/
channelWindow *view::addChannel(imageChannel *channel)
{
    channelWindow *channelWin = new channelWindow(m_panel,channel);
    m_channelSizer->Add(channelWin,wxSizerFlags(0).Expand().Border(wxALL,20));
    updatePanelSize();
    return channelWin;
}


/***************************************************************************/
/*!
 *  Removes a channel and resizes itself
 *
 ****************************************************************************/
void view::removeChannel(channelWindow *window)
{
    m_channelSizer->Detach(window);
    updatePanelSize();
}



////////////////////////////// Private functions ///////////////////////////


/***************************************************************************/
/*!
 *  Helper function: Updates its own size and lays out anew
 *
 ****************************************************************************/
void view::updatePanelSize()
{
    const int stepSize=35;

    m_channelSizer->SetSizeHints(m_panel);
    m_channelSizer->Fit         (m_panel);

    m_channelSizer->Layout();

    wxSize size = m_panel->GetSize();

    int x,y;
    CalcUnscrolledPosition(0,0,&x,&y);
    Scroll(-x/stepSize,-y/stepSize);

    SetScrollbars(stepSize,
        stepSize,
        size.GetWidth() /stepSize,
        size.GetHeight()/stepSize);

}
